/* Interface of dynamic actors in the CT domain.

Copyright (c) 1998-2004 The Regents of the University of California.
All rights reserved.
Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the above
copyright notice and the following two paragraphs appear in all copies
of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

PT_COPYRIGHT_VERSION_2
COPYRIGHTENDKEY

*/

package ptolemy.domains.ct.kernel;

import ptolemy.actor.Actor;
import ptolemy.kernel.util.IllegalActionException;

//////////////////////////////////////////////////////////////////////////
//// CTDynamicActor
/**
   Interface of dynamic actors in the CT domain. Dynamic actors are actors
   that have one or more integrators in their I/O relations.
   Typically, integrators, analog filters, and anything that can be written
   as<BR>
   dx/dt = f(x, u, t)<BR>
   y = g(x, u, t)<BR>
   are dynamic actors.
   <P>
   There is one method defined in this interface, which is
   emitTentativeOutput().
   The tentative output is the output if the current integration step is
   accurate. It may not be the "real" output, if there is an "missed" event
   reported by some event detectors.
   The tentative output is the output of the actor
   after each iteration (but before the states are updated). They are
   emitted for exciting the event generating and the output schedule.

   @author Jie Liu
   @version $Id: CTDynamicActor.java,v 1.26 2004/04/13 03:26:22 cxh Exp $
   @since Ptolemy II 0.2
   @Pt.ProposedRating Green (liuj)
   @Pt.AcceptedRating Green (yuhong)
*/
public interface CTDynamicActor extends Actor{
    /** Implementations of this method should emit the tentative outputs.
     *  @exception IllegalActionException If the data transfer is not
     *       completed.
     */
    public void emitTentativeOutputs() throws IllegalActionException;
}
