/* A base class for all GR actors
   Copyright (c) 2003-2004 The Regents of the University of California.
   All rights reserved.
   Permission is hereby granted, without written agreement and without
   license or royalty fees, to use, copy, modify, and distribute this
   software and its documentation for any purpose, provided that the above
   copyright notice and the following two paragraphs appear in all copies
   of this software.

   IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
   FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
   ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
   THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE.

   THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
   PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
   CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
   ENHANCEMENTS, OR MODIFICATIONS.

   PT_COPYRIGHT_VERSION_2
   COPYRIGHTENDKEY
*/

package ptolemy.domains.gr.kernel;

import ptolemy.domains.gr.lib.ViewScreen2D;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;


//////////////////////////////////////////////////////////////////////////
//// GRActor2D
/**
   A base class for all two-dimensional GR actors. These actors use Diva
   to render two-dimensional scenes in the GR domain. This is an abstract
   class that is never used as a standalone actor in a Ptolemy model.
   Subclasses of this actor include Geometry actors, Transform actors,
   Interaction actors, and the ViewScreen2D display actor.

   @see ptolemy.domains.gr.lib

   @author Steve Neuendorffer
   @version $Id: GRActor2D.java,v 1.11 2004/04/13 03:31:33 cxh Exp $
   @since Ptolemy II 4.0
   @Pt.ProposedRating Red (neuendor)
   @Pt.AcceptedRating Red (cxh)
*/
abstract public class GRActor2D extends GRActor {

    /** Create a new GRActor in the specified container with the specified
     *  name.  The name must be unique within the container or an exception
     *  is thrown. The container argument must not be null, or a
     *  NullPointerException will be thrown.
     *
     *  @param container The container.
     *  @param name The name of this actor within the container.
     *  @exception IllegalActionException If this actor cannot be contained
     *   by the proposed container (see the setContainer() method).
     *  @exception NameDuplicationException If the name coincides with
     *   an entity already in the container.
     */
    public GRActor2D(CompositeEntity container, String name)
            throws IllegalActionException, NameDuplicationException {
        super(container, name);
    }

    ///////////////////////////////////////////////////////////////////
    ////                         public methods                    ////

    ///////////////////////////////////////////////////////////////////
    ////                         protected methods                 ////

    /** Set the view screen that this actor is connected to.
     *  @exception RuntimeException If the given actor is not a
     *  ViewScreen2D.
     */
    protected void _setViewScreen(GRActor actor) {
        if (actor instanceof ViewScreen2D) {
            _viewScreen = (ViewScreen2D)actor;
        } else {
            throw new RuntimeException("Actor " + getClass().getName() +
                    " can only be used with a ViewScreen2D");
        }
    }

    ///////////////////////////////////////////////////////////////////
    ////                         protected variables               ////

    // The actor displaying the scene, if there is one
    protected ViewScreen2D _viewScreen;
}
