// ColorScheme.cpp: ColorScheme class implementation.

//////////////////////////////////////////////////////////////////////



#include "ColorScheme.h"

#include "Utilities.h"



#include <windows.h>





// ============================================================================

// ============================================================================

ColorScheme::ColorScheme()

{

	generateColors();

}



// ============================================================================

void ColorScheme::copyColor(int iColor, RGBTRIPLE * oColor)

{

	oColor->rgbtRed   = oColors[iColor].rgbtRed;

	oColor->rgbtGreen = oColors[iColor].rgbtGreen;

	oColor->rgbtBlue  = oColors[iColor].rgbtBlue;

}



// ============================================================================

RGBTRIPLE * ColorScheme::getColor(int iColor)

{

	return &oColors[iColor];

}



// ============================================================================

void ColorScheme::putColor(int iColor, RGBTRIPLE * oColor)

{

	oColors[iColor].rgbtRed   = oColor->rgbtRed;

	oColors[iColor].rgbtGreen = oColor->rgbtGreen;

	oColors[iColor].rgbtBlue  = oColor->rgbtBlue;

}



// ============================================================================

void ColorScheme::setMaskColor()

{

	oColors[MASK_VALUE].rgbtRed   = 0;

	oColors[MASK_VALUE].rgbtGreen = 0;

	oColors[MASK_VALUE].rgbtBlue  = 0;

}



// ============================================================================

void ColorScheme::generateColors()

{

	int i, j, k;



	// in the default color scheme, colors are shades of gray

	for (i = 0; i < 32; i++)

		for (j = 0; j < 8; j++)

		{

			k = i * 8 + j;

			oColors[k].rgbtRed   = 63 + i * 6;

			oColors[k].rgbtGreen = 63 + i * 6;

			oColors[k].rgbtBlue  = 63 + i * 6;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void SpeciesScheme::generateColors()

{

	for (int i = 0; i < 256; i++)

	{

		oColors[i].rgbtRed   = 0;

		oColors[i].rgbtGreen = 0;

		oColors[i].rgbtBlue  = 0;

	}



	// absence value

	oColors[ABSENCE].rgbtRed   = 63;

	oColors[ABSENCE].rgbtGreen = 127;

	oColors[ABSENCE].rgbtBlue  = 63;

	

	// presence value

	oColors[PRESENCE].rgbtRed   = 127;

	oColors[PRESENCE].rgbtGreen = 255;

	oColors[PRESENCE].rgbtBlue  = 127;



	// undefined value

	oColors[MISSING_VALUE].rgbtRed   = 127;

	oColors[MISSING_VALUE].rgbtGreen = 127;

	oColors[MISSING_VALUE].rgbtBlue  = 127;



	// mask value

	setMaskColor();

}



// ============================================================================

// ============================================================================

void TemperatureScheme::generateColors()

{

	int i, j, k;



	for (i = 0; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 63 + i * 12;

			oColors[k].rgbtGreen = 7 * i;

			oColors[k].rgbtBlue  = 63 + (16 - i) * 12;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void PrecipitationScheme::generateColors()

{

	int i, j, k;



	for (i = 0; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 7 * i;

			oColors[k].rgbtGreen = 63 + i * 12;

			oColors[k].rgbtBlue  = 63 + i * 12;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void SnowfallScheme::generateColors()

{

	int i, j, k;



	// brown colors

	for (i = 0; i < 2; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 127 + i * 32;

			oColors[k].rgbtGreen = 127 + i * 32;

			oColors[k].rgbtBlue  = 31;

		}



	// blue/white colors

	for (i = 2; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 95 + i * 10;

			oColors[k].rgbtGreen = 95 + i * 10;

			oColors[k].rgbtBlue  = 255;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void SolarRadiationScheme::generateColors()

{

	int i, j, k;



	for (i = 0; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 63 + i * 12;

			oColors[k].rgbtGreen = 63 + i * 12;

			oColors[k].rgbtBlue  = i * 12;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void ElevationScheme::generateColors()

{

	int i, j, k;



	// green colors

	for (i = 0; i < 2; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 63;

			oColors[k].rgbtGreen = 160 - (i * 16);

			oColors[k].rgbtBlue  = 63;

		}



	// brown colors

	for (i = 2; i < 6; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 127 + (i - 2) * 8;

			oColors[k].rgbtGreen = 127 + (i - 2) * 8;

			oColors[k].rgbtBlue  = 31;

		}



	// white colors

	for (i = 6; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 95 + i * 10;

			oColors[k].rgbtGreen = 95 + i * 10;

			oColors[k].rgbtBlue  = 255;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void SlopeScheme::generateColors()

{

	int i, j, k;



	// gray colors

	for (i = 0; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 191 - i * 6;

			oColors[k].rgbtGreen = 191 - i * 6;

			oColors[k].rgbtBlue  = 191 - i * 6;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void VegetationScheme::generateColors()

{

	int i, j, k;



	for (i = 0; i < 16; i++)

		for (j = 0; j < 16; j++)

		{

			k = i * 16 + j;

			oColors[k].rgbtRed   = 63;

			oColors[k].rgbtGreen = 63 + i * 8;

			oColors[k].rgbtBlue  = 63;

		}



	setMaskColor();

}



// ============================================================================

// ============================================================================

void CustomScheme::generateColors()

{

	setMaskColor();

}



// ============================================================================

// ============================================================================

// ============================================================================

ColorSchemes::ColorSchemes()

{

	oSchemes[0] = new ColorScheme;

	oSchemes[1] = new SpeciesScheme;

	oSchemes[2] = new TemperatureScheme;

	oSchemes[3] = new PrecipitationScheme;

	oSchemes[4] = new SnowfallScheme;

	oSchemes[5] = new SolarRadiationScheme;

	oSchemes[6] = new ElevationScheme;

	oSchemes[7] = new SlopeScheme;

	oSchemes[8] = new VegetationScheme;

	oSchemes[9] = new CustomScheme;



	strcpy(sLayerType[0], "Default");

	strcpy(sLayerType[1], "Species");

	strcpy(sLayerType[2], "Temperature");

	strcpy(sLayerType[3], "Precipitation");

	strcpy(sLayerType[4], "Snowfall");

	strcpy(sLayerType[5], "SolarRadiation");

	strcpy(sLayerType[6], "Elevation");

	strcpy(sLayerType[7], "Slope");

	strcpy(sLayerType[8], "Vegetation");

	strcpy(sLayerType[9], "Custom");

}        



// ============================================================================

ColorScheme * ColorSchemes::getScheme(char * sLayerTypeProvided)

{

	for (int i = 0; i < NumOfColorSchemes; i++)

		if (strcmp(sLayerTypeProvided, sLayerType[i]) == 0)

			return oSchemes[i];



	// no color scheme found: return default

	return oSchemes[0];

}



// ============================================================================

// ============================================================================

