/**
 *    '$RCSfile: EML2ReturnFieldsHandler.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/05/03 20:50:43 $'
 *   '$Revision: 1.3 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.seek.ecogrid;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.helpers.DefaultHandler;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Enumeration;

import java.util.Vector;
import java.util.Stack;

/**
 * This class will parse an xml segment from ecogrid search resultset and
 * get the return fields value for document. We need is document's title and
 * list of entity name
 * @author Jing Tao
 */
public class EML2ReturnFieldsHandler implements ContentHandler
{
  private Stack elementStack = null;
  private String recordTitle = null;
  private Vector entityNameVector = null;
  private String paramName = null;
  private String titleString = null;
  private String entityString = null;
  // global for accumulating characters in SAX parser
  private String accumulatedCharacters = null;
  private boolean gotTitle  = false;

  private static final String PARAM = "param";
  private static final String NAME  = "name";


  /**
   *  A contruct will be used in another thread
   * @param resultsXMLStream InputStream
   * @param titleStr  String  the name indicate the param element is a title
   * @param entityStr String  the name indicate the param element is a entity
   */
  public EML2ReturnFieldsHandler(String titleStr, String entityStr)
                                 throws SAXException
  {
    if (titleStr == null || entityStr == null
        || titleStr.equals("") || entityStr.equals(""))
    {
      throw new SAXException("title or entity tag is not specifiy" +
                             " in EML2ReturnFieldHanlder");
    }
    entityNameVector = new Vector();
    titleString      = titleStr;
    entityString     = entityStr;

  }


  /**
   * Get the title for the document, null will be returned if no title found
   * @return String
   */
  public String getRecordTitle()
  {
    return recordTitle;
  }


  /**
   * Get the entity name vector. emty vector will be returned if no entity found
   * @return Vector
   */
  public Vector getEntityNameVector()
  {
    return entityNameVector;
  }




  /**
   * SAX handler callback that is called upon the start of an
   * element when parsing an XML document.
   */
  public void startElement(String uri, String localName,
                           String qName, Attributes atts) throws SAXException
  {
    if (localName.equalsIgnoreCase(PARAM))
    {
      paramName = atts.getValue(NAME);
    }
    else
    {
      paramName = null;
    }
    // when hit new element, the accumulatedCharater need be reset
    accumulatedCharacters = "";

  }

  /**
   * SAX handler callback that is called for character content of an
   * element when parsing an XML document.
   */
  public void characters(char ch[], int start, int length)
  {
    String inputString = new String(ch, start, length);
    if (inputString != null)
    {
      accumulatedCharacters = accumulatedCharacters + inputString;
    }

  }




  /**
   * SAX handler callback that is called upon the end of an
   * element when parsing an XML document.
   */
  public void endElement(String uri, String localName,
                         String qName) throws SAXException
  {
    if (localName.equalsIgnoreCase(PARAM))
    {
      String inputString = accumulatedCharacters.trim();
      // get the first title value
      if (paramName != null && paramName.indexOf(titleString) !=-1
          && !inputString.equals("") && !gotTitle)
      {
        recordTitle = inputString;
        gotTitle    = true;
      }
      // get the entityname and put into vector
      if (paramName != null && paramName.indexOf(entityString) != -1
          && !inputString.equals(""))
      {
        entityNameVector.addElement(inputString);
      }
    }

  }



  /**
   * SAX handler callback that is called when an XML document
   * is initially parsed.
   */
  public void startDocument() throws SAXException {

  }

  /** Unused SAX handler */
  public void endDocument() throws SAXException {

  }

  /** Unused SAX handler */
  public void ignorableWhitespace(char[] cbuf, int start, int len) {
  }

  /** Unused SAX handler */
  public void skippedEntity(String name) throws SAXException {
  }

  /** Unused SAX handler */
  public void processingInstruction(String target, String data) throws
      SAXException {
  }

  /** Unused SAX handler */
  public void startPrefixMapping(String prefix, String uri) throws SAXException {
  }

  /** Unused SAX handler */
  public void endPrefixMapping(String prefix) throws SAXException {
  }

  /** Unused SAX handler */
  public void setDocumentLocator(Locator locator) {
  }
}
