/**

 *    '$RCSfile: QueryAction.java,v $'

 *

 *     '$Author: tao $'

 *       '$Date: 2004/05/01 01:39:52 $'

 *   '$Revision: 1.3 $'

 *

 *  For Details: http://kepler.ecoinformatics.org

 *

 * Copyright (c) 2004 The Regents of the University of California.

 * All rights reserved.

 *

 * Permission is hereby granted, without written agreement and without

 * license or royalty fees, to use, copy, modify, and distribute this

 * software and its documentation for any purpose, provided that the

 * above copyright notice and the following two paragraphs appear in

 * all copies of this software.

 *

 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN

 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY

 * OF SUCH DAMAGE.

 *

 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY

 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,

 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

 */



package org.ecoinformatics.seek.ecogrid;



import java.awt.event.ActionEvent;

import java.net.URL;



import javax.swing.AbstractAction;



import org.ecoinformatics.ecogrid.QueryType;

import org.ecoinformatics.ecogrid.ResultsetType;

import org.ecoinformatics.ecogrid.client.EcogridQueryClient;

import org.ecoinformatics.util.DebugPrinter;



/**

 * This class will access ecogrid nodes: send the query and get the resultset

 * @author Tao

 */



public class QueryAction extends AbstractAction

{

  private QueryType                  query = null;

  private String                       url = null;

  private ResultsetType            results = null;

  private EcogridQueryClient ecogridClient = null;

  private DebugPrinter dbg                 = null;

  private String error                     = null;



  /**

   * Contructor of QueryAction

   * @param myQuery QueryType

   * @param url Vector

   */

  public QueryAction(QueryType myQuery, String myUrl)

  {

    query = myQuery;

    url   = myUrl;

    dbg   = DebugPrinter.getInstance();

  }



 /**

  * The todo Implementation of abstract method. It will search ecogrid site

  *

  * @param e ActionEvent

  */

 public void actionPerformed(ActionEvent e)

 {

   if( query == null || url == null)

   {

     error = "Query is null or search url is null";

     dbg.print(error, 2);

     return;

   }

   try

   {

     // create a client object

     ecogridClient = new EcogridQueryClient(new URL(url));

     // create ecogrid query interface porty type

     ecogridClient.createEcoGridQueryLevelOnePortType();

     // query ecogrid nodes

     results = ecogridClient.query(query);

   }

   catch(Exception ee)

   {

     error = ee.getMessage();

     dbg.print(error, 2);

   }

   // if can't get a result, i.e, results equals null, we will conisder

   // there is some error happend.

   if (results == null)

   {

     error = "Couldn't find any results after querying";

   }

 }//actionPerformed



 /**

  * Method to get the resultset after query

  * @return ResultsetType

  */

 public ResultsetType getResultSet()

 {

   return this.results;

 }//getResultSet



 /**

  * Method to return the error message when excute query action

  * @return String

  */

 public String getError()

 {

   return error;

 }

}//QueryAction

