/**

 *    '$RCSfile: SearchScope.java,v $'

 *

 *     '$Author: tao $'

 *       '$Date: 2004/05/04 01:16:36 $'

 *   '$Revision: 1.2 $'

 *

 *  For Details: http://kepler.ecoinformatics.org

 *

 * Copyright (c) 2004 The Regents of the University of California.

 * All rights reserved.

 *

 * Permission is hereby granted, without written agreement and without

 * license or royalty fees, to use, copy, modify, and distribute this

 * software and its documentation for any purpose, provided that the

 * above copyright notice and the following two paragraphs appear in

 * all copies of this software.

 *

 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN

 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY

 * OF SUCH DAMAGE.

 *

 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY

 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,

 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

 */





package org.ecoinformatics.seek.ecogrid;



import edu.ucsb.nceas.configxml.ConfigXML;



import java.util.Vector;



import org.ecoinformatics.seek.ecogrid.exception.

                                          NoMetadataSpecificationClassException;

import org.ecoinformatics.seek.ecogrid.exception.NoSearchEndPointException;

import org.ecoinformatics.seek.ecogrid.exception.

                                              NULLEcogridConfigurationException;

import org.ecoinformatics.seek.ecogrid.exception.NULLSearchNamespaceException;

import org.ecoinformatics.util.DebugPrinter;

import org.ecoinformatics.util.Util;

/**

 * <p>Title: SearchScope</p>.

 * <p>Description: This class represents a search scope:

*      namespace - the documents's namespace need to be searched

*      metadataSpecificationClass - the metadata class correspond to the

*                                    namespace

*      endPoints - the url list which have those documents

*  </p>

 * <p>Copyright: Copyright (c) 2004</p>

 * <p>Company: UCSB </p>

 * @author tao

 * @version 1.0

 */



public class SearchScope

{

  private String namespace                                     = null;

  private ConfigXML config                                     = null;

  private MetadataSpecificationInterface metadataSpecification = null;

  private Vector endPointsVector                               = null;

  private DebugPrinter dbg                                     = null;



  // constant

  private String METADATACLASS = "metadataSpecificationClass";

  private String NAMESPACE     = "namespace";

  /**

   * Constructor

   * @param nameSpace String  the namesapce need to be searched

   * @param endPoints Vector  url need to go searching

   * @throws NULLSearchNamespaceException

   * @throws NoMetadataSpecificationClassException

   */

  public SearchScope(String nameSpace, ConfigXML conf, Vector endPoints)

                     throws NULLSearchNamespaceException,

                            NULLEcogridConfigurationException,

                            NoMetadataSpecificationClassException,

                            NoSearchEndPointException

  {

    dbg = DebugPrinter.getInstance();

    if (nameSpace == null || nameSpace.equals(""))

    {

      throw new NULLSearchNamespaceException("Search namespace is null");

    }

    this.namespace = nameSpace;

    dbg.print("The namespace in SearchScope is " + namespace, 2);

    if (conf == null)

    {

      throw new NULLEcogridConfigurationException("Ecogrid config is null");

    }

    this.config = conf;

    // create a metadata specification class object

    metadataSpecification = generateMetadataSpecification();

    metadataSpecification.setConfigure(config);

    if (endPoints == null || endPoints.isEmpty())

    {

      throw new NoSearchEndPointException("No end points in Search Scope");

    }

    endPointsVector = endPoints;

  }//SearchScope



  /*

   * This method will create a concrete class object for

   * MetadataSpecificationInterface from configure file base on given namesapce

   */

  private MetadataSpecificationInterface generateMetadataSpecification()

                                   throws NoMetadataSpecificationClassException

  {

     String error = "Could NOT find the class name for metadata specification "+

                   "in ecogrid configure file";



     MetadataSpecificationInterface metadataSpec = null;

     if (namespace == null || namespace.equals(""))

     {

       throw new NoMetadataSpecificationClassException("namespace is null " +

                          "and couldn't find the metadataspecification class");

     }

     // get full name of MetadataSpecification class name base on namespace

     // from configuration file

     String xpath  = "//" + METADATACLASS + "[@" + NAMESPACE + "='" +

                     namespace + "']";



     Vector metadataSpecificationClassNameVec = null;

     try

     {

       metadataSpecificationClassNameVec = config.getValuesForPath(xpath);

     }

     catch (Exception e)

     {



       throw new NoMetadataSpecificationClassException(error);

     }

     //check if the return vector is null or not

     if (metadataSpecificationClassNameVec == null ||

         metadataSpecificationClassNameVec.isEmpty())

     {



       throw new NoMetadataSpecificationClassException(error);

     }

     String className = (String)metadataSpecificationClassNameVec.elementAt(0);

     dbg.print("The class name is " + className, 2);



     try

     {

       metadataSpec = (MetadataSpecificationInterface)

                                                  Util.createObject(className);

     }

     catch (Exception ee)

     {

       throw new NoMetadataSpecificationClassException("Couldn't get the " +

                            "object for this class " + className);

     }

     return metadataSpec;

  }



  /**

   * Method to get namespace

   * @return String the namespace

   */

  public String getNamespace()

  {

    return namespace;

  }



  /**

   * Method to get metadata specification class

   * @return MetadataSpecificationInterface

   */

  public MetadataSpecificationInterface getMetadataSpecification()

  {

    return this.metadataSpecification;

  }



  /**

   * Method go get endponts (url)

   * @return Vector

   */

  public Vector getEndPoints()

  {

    return endPointsVector;

  }

}//SearchScope

