/**
 *    '$RCSfile: Util.java,v $'
 *
 *     '$Author: tao $'
 *       '$Date: 2004/08/11 01:16:05 $'
 *   '$Revision: 1.6 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */


package org.ecoinformatics.util;

import java.lang.*;
import java.io.*;
import java.util.*;
import java.net.URL;

import org.apache.xpath.XPathAPI;
import org.ecoinformatics.seek.ecogrid.ResultRecord;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.ext.DeclHandler;
import org.xml.sax.ext.LexicalHandler;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.AttributesImpl;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.XMLReaderFactory;
import org.xml.sax.InputSource;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Attr;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.DocumentType;
import org.apache.xerces.dom.DocumentTypeImpl;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import javax.servlet.ServletException;

import com.arbortext.catalog.*;

import edu.ucsb.nceas.configxml.*;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;

import javax.xml.transform.*;
import javax.xml.transform.stream.*;

import gnu.regexp.RE;
import gnu.regexp.REException;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.parsers.SAXParser;


/**
 * This is a utility class.  Any static methods that may be needed
 * by multiple classes should be put here.
 */
public class Util
{
  private static final String idDelimiter = "@";
  private static String pipelineLocation = null;
  private static final String DBUSERNAME = "sa";
  private static final String DBPASSWORD = "";
  private static final String DBNAMEPATH = "data/hsql/hsqldb";


  static public String getIdDelimiter()
  {
    return idDelimiter;
  }

  /**
   * creates an object of a type className.  this is used for instantiating
   * plugins.
   */
  public static Object createObject(String className)
                throws InstantiationException,
                       IllegalAccessException,
                       ClassNotFoundException
  {
    Object object = null;
    try
    {
      Class classDefinition = Class.forName(className);
      object = classDefinition.newInstance();
    }
    catch (InstantiationException e)
    {
      throw new InstantiationException("Error instantiating class.");
    }
    catch (IllegalAccessException e)
    {
      throw new IllegalAccessException("Error accessing class.");
    }
    catch (ClassNotFoundException e)
    {
      throw new ClassNotFoundException("Class " + className + " not found.");
    }
    return object;
  }

  /**
   * returns true if the inputstream's first 6 bytes = "&lt;? xml"
   */
  public static boolean hasXMLHeader(InputStream is) throws IOException
  {
    StringBuffer sb = new StringBuffer();
    for(int i=0; i<6; i++)
    {
      sb.append((char)is.read());
    }

    String s = sb.toString();
    if(s.indexOf("<?xml") != -1)
    {
      return true;
    }

    return false;
  }

  /**
   * looks through the child nodes of n and returns the first child with name
   * @param name the name of the child node to search for
   * @param n the node whose children we are searching
   * @return the found node or null if the node is not found.
   */
  public static Node getChildNodeWithName(String name, Node n)
  {
    NodeList nl = n.getChildNodes();
    for(int i=0; i<nl.getLength(); i++)
    {
      Node c = nl.item(i);
      if(c.getNodeName().equals(name))
      {
        return c;
      }
    }
    return null;
  }

  /**
   * searches for a child of n called childNodeName.  If it is found, the
   * nodeValue() of the child is returned.
   * If it is not found, null is returned.  See the Xerces documentation
   * for class Node for return values.
   * @param childNodeName the child node to search for
   * @param n the node to search through.
   */
  public static String getChildNodeValue(String childNodeName, Node n)
  {
    Node child = getChildNodeWithName(childNodeName, n);
    return child.getNodeValue();
  }

  /**
   * returns the content of the path in an xml file
   */
  public static NodeList getPathContent(File f, String path) throws Exception
  {
    return getPathContent(new FileInputStream(f), path,
                          new Config().getCatalogPath());
  }

  /**
   * returns the content of the path in an xml file
   */
  public static NodeList getPathContent(File f, String path, String catalogPath)
         throws Exception
  {
    return getPathContent(new FileInputStream(f), path, catalogPath);
  }

  /**
   * returns the path nodelist using the default catalog path
   */
  public static NodeList getPathContent(InputStream is, String path)
    throws Exception
  {
    return getPathContent(is, path, getCatalogPath());
  }

  /**
   * Gets the content of a path in an xml file
   */
  public static NodeList getPathContent(InputStream is, String path,
                                        String catalogPath)
         throws Exception
  {
    if(is == null)
    {
      return null;
    }

    DOMParser parser = new DOMParser();
    InputSource in;

    CatalogEntityResolver cer = new CatalogEntityResolver();
    try
    {
      Catalog myCatalog = new Catalog();
      myCatalog.loadSystemCatalogs();
      myCatalog.parseCatalog(catalogPath);
      cer.setCatalog(myCatalog);
    }
    catch (Exception e)
    {
      throw e;
    }

    parser.setEntityResolver(cer);

    in = new InputSource(is);

    try
    {
      parser.parse(in);
    }
    catch(Exception e1)
    {
      throw new Exception("parse threw: " + e1.toString());
    }

    Document doc = parser.getDocument();

    try
    {
      NodeList docNodeList = XPathAPI.selectNodeList(doc, path);
      return docNodeList;
    }
    catch(Exception e)
    {
      throw new Exception("parse threw: " + e.toString());
    }
  }

   /**
   * strips all of the path information off of a package file just leaving the
   * file id.  ex. /home/berkley/package/berkley.1.1.package/berkley.1.1
   * become berkley.1.1
   */
  public static String getIdFromFileName(String filename)
  {
    return filename.substring(filename.lastIndexOf("/") + 1, filename.length());
  }

  /**
   * get the Catalog object
   */
  public static CatalogEntityResolver getCatalog() throws FileNotFoundException, Exception
  {
    CatalogEntityResolver cer = new CatalogEntityResolver();
    Catalog myCatalog = new Catalog();
    myCatalog.loadSystemCatalogs();
    String catalogPath = new Config().getCatalogPath();
    myCatalog.parseCatalog(catalogPath);
    cer.setCatalog(myCatalog);

    return cer;
  }

  /**
   * get the path to the catalog
   */
  public static String getCatalogPath() throws FileNotFoundException,
                edu.ucsb.nceas.configxml.exception.ElementNotFoundException,
                Exception
  {
    return new Config().getCatalogPath();
  }

  /**
   * substitutes subval for val in text
   */
  public static String substitute(String text, String val, String subval)
         throws REException
  {
    RE regexp = new RE(val);
    text = regexp.substituteAll(text, subval);
    return text;
  }

  /**
   * returns a Document using the default catalog path
   */
  public static Document getDoc(File file) throws SAXException, Exception
  {
    return getDoc(new FileInputStream(file), getCatalogPath());
  }

  /**
   * returns a Document from a FileInputStream using the default catalog path
   */
  public static Document getDoc(InputStream is) throws SAXException, Exception
  {
    return getDoc(is, getCatalogPath());
  }

  /**
   *
   */
  public static Document getDoc(File file, String catalogPath)
    throws SAXException, Exception
  {
    return getDoc(new FileInputStream(file), catalogPath);
  }

  /**
   * returns an inputstream that is an exact copy of the inputstream passed
   * in.  The returned copy is a different stream so that the two can be
   * manipulated seperately.
   */
  public static InputStream copyStream(InputStream is)
  {
    try
    {
      int buffersize = (new Config()).getInputStreamBufferSize();
      byte[] b = new byte[buffersize];
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      int bytesread = is.read(b, 0, buffersize);
      while(bytesread != -1)
      {
        baos.write(b, 0, bytesread);
        bytesread = is.read(b, 0, buffersize);
      }

      ByteArrayInputStream bais = new ByteArrayInputStream(baos.toByteArray());
      return bais;
    }
    catch(IOException ioe)
    {
      throw new RuntimeException("Error copying stream.");
    }
  }

  /**
   * parses file with the dom parser and returns a dom Document
   * @param fs the FileInputStream to create the document from
   * @param catalogPath the path to the catalog where the files doctype info
   * can be found.
   */
  public static Document getDoc(InputStream is, String catalogPath)
    throws SAXException, Exception
  {
    DOMParser parser = new DOMParser();
    Document doc;
    InputSource in;
    CatalogEntityResolver cer = new CatalogEntityResolver();
    try
    {
      Catalog myCatalog = new Catalog();
      myCatalog.loadSystemCatalogs();
      myCatalog.parseCatalog(catalogPath);
      cer.setCatalog(myCatalog);
    }
    catch (Exception e)
    {
      throw new Exception("Error creating catalog.");
    }

    parser.setEntityResolver(cer);

    in = new InputSource(is);
    try
    {
      parser.parse(in);
    }
    catch(Exception e1)
    {
      throw new Exception("Error Parsing document.");
    }
    //get the DOM rep of the document without triples
    doc = parser.getDocument();

    return doc;
  }

  /**
   * Set up a DOM parser for reading an XML document
   *
   * @return a DOM parser object for parsing
   */
  public static DocumentBuilder createDomParser() throws Exception
  {
    DocumentBuilder parser = null;

    try
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      factory.setNamespaceAware(true);
      parser = factory.newDocumentBuilder();
      if (parser == null)
      {
        throw new Exception("Could not create Document parser in " +
          "Util.createDomParser");
      }
    }
    catch (ParserConfigurationException pce)
    {
      throw new Exception("Could not create Document parser in " +
        "Util.createDomParser.");
    }

    return parser;
  }

  /**
   * returns the total size in bytes of all files in a directory and its
   * subdirectories
   * @param directoryPath the directory to find the size of
   */
  public static long getDirectorySize(String directoryPath)
          throws Exception
  {
    return getDirectorySize(directoryPath, 0);
  }

  /**
   * returns the total size in bytes of all files in a directory and its
   * subdirectories
   * @param directoryPath the directory to find the size of
   * @param size the size to start out with.  Usually this should be 0
   */
  private static long getDirectorySize(String directoryPath, long size)
          throws Exception
  {
    long length = 0;
    File root = new File(directoryPath);
    if(!root.isDirectory())
    {
      throw new Exception("The path " + root.getAbsolutePath() +
        " specified is not a directory");
    }
    if(!root.exists())
    {
      throw new Exception("The path " + root.getAbsolutePath() +
        " does not exist.");
    }
    if(!root.canRead())
    {
      throw new Exception("Invalid read permissions on the path " +
        root.getAbsolutePath());
    }

    String[] listing = root.list();
    Vector dirs = new Vector();
    //add up the file sizes
    for(int i=0; i<listing.length; i++)
    {
      File f = new File(directoryPath + "/" + listing[i]);
      if(f.isFile())
      {
        length += f.length();
      }
      else if(f.isDirectory())
      {
        dirs.addElement(f);
      }
    }

    //recurse into each directory
    for(int i=0; i<dirs.size(); i++)
    {
      File f = (File)dirs.elementAt(i);
      length = getDirectorySize(f.getAbsolutePath(), length);
    }
    return length + size;
  }

  /**
   * deletes a directory tree, not including the child of directoryPath.
   * if directoryPath == "/temp/cache"  all of the contents of 'cache'
   * will be deleted but not 'cache' itself
   * @param directoryPath the directory to delete the contents of
   */
  public static void deleteDirectoryTree(String directoryPath)
                    throws Exception
  {
    long length = 0;
    File root = new File(directoryPath);
    if(!root.isDirectory())
    {
      throw new Exception("The path " + root.getAbsolutePath() +
        " specified is not a directory");
    }
    if(!root.exists())
    {
      throw new Exception("The path " + root.getAbsolutePath() +
        " does not exist.");
    }
    if(!root.canWrite())
    {
      throw new Exception("Invalid write permissions on the path " +
        root.getAbsolutePath());
    }

    String[] listing = root.list();
    Vector dirs = new Vector();
    //delete the files
    for(int i=0; i<listing.length; i++)
    {
      File f = new File(directoryPath + "/" + listing[i]);
      if(f.isFile())
      {
        if(!f.delete())
        {
          throw new Exception("Could not delete " +
            f.getAbsolutePath());
        }
      }
      else if(f.isDirectory())
      {
        dirs.addElement(f);
      }
    }

    //recurse into each directory
    for(int i=0; i<dirs.size(); i++)
    {
      File f = (File)dirs.elementAt(i);
      deleteDirectoryTree(f.getAbsolutePath());
      if(!f.delete())
      { //delete this directory
        throw new Exception("Could not delete " +
          f.getAbsolutePath());
      }
    }
  }

  /**
   * returns true if a document has "&lt;?xml" at the beginning of it
   */
  public static boolean isXMLDoc(File f)
    throws FileNotFoundException, IOException
  {
    FileReader fr = new FileReader(f);
    char[] c = new char[200];
    fr.read(c, 0, 200);
    String s = new String(c);
    if(s.indexOf("<?xml") != -1)
    {
      return true;
    }
    fr.close();

    return false;
  }

  /**
   * returns a DOM Document from an xml string
   */
  public static Document getDoc(String s)
    throws javax.xml.parsers.ParserConfigurationException,
           SAXException, IOException
  {
    StringReader sr = new StringReader(s);
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    DocumentBuilder db = dbf.newDocumentBuilder();
    Document d = db.parse(new InputSource(sr));
    return d;
  }

  /**
   * transforms an xml document using the specified xsl stylesheet
   * @param xml the document to transform
   * @param xsl the stylesheet to use
   */
  public static String transformXML(Reader xml, Reader xsl)
    throws TransformerException
  {
    if( xsl != null )
    {
      TransformerFactory tFactory = TransformerFactory.newInstance();
      // Get the XML input document and the stylesheet, both in the servlet
      // engine document directory.
      Source xmlSource = new StreamSource(xml);
      Source xslSource = new StreamSource(xsl);
      // Generate the transformer.
      Transformer transformer = tFactory.newTransformer(xslSource);
      // Perform the transformation, sending the output to the response.
      StringWriter outputWriter;
      transformer.transform(xmlSource,
              new StreamResult(outputWriter = new StringWriter()));
      return outputWriter.getBuffer().toString();
    }

    throw new TransformerException("XSL stylesheet is null.");
  }

  /**
   * returns a string representation of a stack trace from the exception e
   * @param e the exception to return the stack trace from
   */
  static public String getStackTrace(Exception e)
  {
    StringWriter sw = new StringWriter();
    e.printStackTrace(new PrintWriter(sw));
    return sw.toString();
  }

  /**
   * Normalizes the given string.
   *
   * @param s  string to normize
   * @return   Description of the Returned Value
   */
  public static String normalize(String s)
  {
    StringBuffer str = new StringBuffer();
    char oxc = (char)Integer.decode("0xC").intValue();

    int len = (s != null) ? s.length() : 0;
    for (int i = 0; i < len; i++)
    {
      char ch = s.charAt(i);
      switch (ch)
      {
        case '<':
        {
          str.append("&lt;");
          break;
        }
        case '>':
        {
          str.append("&gt;");
          break;
        }
        case '&':
        {
          str.append("&amp;");
          break;
        }
        case '"':
        {
          str.append("&quot;");
          break;
        }
        case '\u000c':
        {
          str.append("");
          break;
        }
        /*case '\r':
        case '\t':
        case '\n':
        {
          if (false)
          {
              str.append("&#");
              str.append(Integer.toString(ch));
              str.append(';');
              break;
          }
          // else, default append char
          break;
        }*/
        default:
        {
          str.append(ch);
        }
      }
    }

    return str.toString();
  }

  /**
    * Set up a SAX parser for reading an XML document
    *
    * @param contentHandler  object to be used for parsing the content
    * @param errorHandler    object to be used for handling errors
    * @return                a SAX XMLReader object for parsing
    */
   public static XMLReader createSaxParser(ContentHandler contentHandler,
           ErrorHandler errorHandler)
   {
       XMLReader parser = null;
       DebugPrinter dbg = DebugPrinter.getInstance();
       // Set up the SAX document handlers for parsing
       try {

           // Get an instance of the parser
           SAXParserFactory spfactory = SAXParserFactory.newInstance();
           SAXParser saxp = spfactory.newSAXParser();
           parser = saxp.getXMLReader();

           if (parser != null) {
               parser.setFeature("http://xml.org/sax/features/namespaces",
                   true);
               dbg.print("Parser created is: " +
                         parser.getClass().getName(), 2);
           } else {
               dbg.print("Unable to create SAX parser!", 2);
           }

           // Set the ContentHandler to the provided object
           if (null != contentHandler) {
               parser.setContentHandler(contentHandler);
           } else {
               dbg.print("No content handler for SAX parser!", 2);
           }

           // Set the error Handler to the provided object
           if (null != errorHandler) {
               parser.setErrorHandler(errorHandler);
           }
       } catch (Exception e) {
           dbg.print("Failed to create SAX parser:\n" + e.toString(), 2);
       }

       return parser;
   }

  /**
   * Method to transfer a array of ResultRecord to a Vector
   * @param source  the source array of ResultRecord
   * @param dest    the destation vector the element will be added.
   */
  public static void transformResultRecordArrayToVector(ResultRecord[] source,
                                                   Vector dest)
  {
    if (source == null || dest == null)
    {
      return;
    }
    int size = source.length;
    for (int i = 0; i < size; i++)
    {
      ResultRecord record = source[i];
      dest.add(record);
    }
  }

  public static Connection getDBConnection() throws SQLException,
                                           ClassNotFoundException
  {

      Connection conn = null;
      // Load the HSQL Database Engine JDBC driver
      // hsqldb.jar should be in the class path or made part of the current jar
      Class.forName("org.hsqldb.jdbcDriver");

      // connect to the database.   This will load the db files and start the
      // database if it is not alread running.
      // db_file_name_prefix is used to open or create files that hold the state
      // of the db.
      // It can contain directory names relative to the
      // current working directory
      conn = DriverManager.getConnection("jdbc:hsqldb:" + DBNAMEPATH,
                                           DBUSERNAME, DBPASSWORD);

      return conn;
  }

}
