/* A touchgraph ontolgy display actor.



@Copyright (c) 2002-2003 The Regents of the University of California.

All rights reserved.



Permission is hereby granted, without written agreement and without

license or royalty fees, to use, copy, modify, and distribute this

software and its documentation for any purpose, provided that the

above copyright notice and the following two paragraphs appear in all

copies of this software.



IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF

THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF

SUCH DAMAGE.



THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF

CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,

ENHANCEMENTS, OR MODIFICATIONS.



												PT_COPYRIGHT_VERSION 2

												COPYRIGHTENDKEY



*/



package org.geon;



import ptolemy.actor.TypedAtomicActor;

import ptolemy.actor.TypedIOPort;

import ptolemy.data.ArrayToken;

import ptolemy.data.BooleanToken;

import ptolemy.data.IntToken;

import ptolemy.data.StringToken;

import ptolemy.data.UnsignedByteToken;

import ptolemy.data.Token;

import ptolemy.data.expr.FileParameter;

import ptolemy.data.expr.Parameter;

import ptolemy.data.type.ArrayType;

import ptolemy.data.type.BaseType;

import ptolemy.kernel.CompositeEntity;

import ptolemy.kernel.util.*;



import java.io.*;

import javax.swing.*;



import java.awt.event.*;

import org.ecoinformatics.seek.ontobrowser.*;

import org.ecoinformatics.seek.growl.*;

import org.ecoinformatics.seek.growl.editor.*;



//////////////////////////////////////////////////////////////////////////

//// OntoBrowser

/**

This actor loads a touchgraph ontology display application (GROWL).



@author  skrivov

@version $Id: OntoBrowser.java,v 1.5 2004/07/07 20:02:09 jaeger Exp $

@since Ptolemy II 3.0.2

*/

public class OntoBrowser extends TypedAtomicActor {



	/** Construct an actor with the given container and name.

	 *  @param container The container.

	 *  @param name The name of this actor.

	 *  @exception IllegalActionException If the actor cannot be contained

	 *   by the proposed container.

	 *  @exception NameDuplicationException If the container already has an

	 *   actor with this name.

	 */

	public OntoBrowser(CompositeEntity container, String name)

				throws IllegalActionException, NameDuplicationException {

			super(container, name);



			ontologyFile = new FileParameter(this, "ontology file");

	}



	///////////////////////////////////////////////////////////////////

	////                     ports and parameters                  ////



	/** The ontology file name to be loaded.  If empty just load the application.

	 */

	public FileParameter ontologyFile;



	///////////////////////////////////////////////////////////////////

	////                         public methods                    ////



	/** If the specified attribute is <i>ontologyFile</i> and it's an empty

	 *  string. Load the application with no ontology.

	 *  @param attribute The attribute that has changed.

	 *  @exception IllegalActionException.

	 */

	public void attributeChanged(Attribute attribute)

			throws IllegalActionException {

		if (attribute == ontologyFile) {

			// NOTE: We do not want to close the file if the file

			// has not in fact changed.  We check this by just comparing

			// name, which is not perfect...

			if (_previousOntoFile != null

					&& !ontologyFile.getExpression().equals(_previousOntoFile)) {

				_previousOntoFile = ontologyFile.getExpression();

				// Do not load an ontology if the ontologyFile is blank.

				if (ontologyFile.getExpression().trim().equals("")) {

					_noOntoFile = true;

				} else {

					_noOntoFile = false;

				}

			}

		} else {

			super.attributeChanged(attribute);

		}

	}



	/** Clone the actor into the specified workspace.

	 *  @return A new actor.

	 *  @exception CloneNotSupportedException If a derived class contains

	 *   an attribute that cannot be cloned.

	 */

	public Object clone(Workspace workspace)

			throws CloneNotSupportedException {

		OntoBrowser newObject = (OntoBrowser)super.clone(workspace);

		newObject._noOntoFile = true;

		return newObject;

	}



	/** Load the .

	 *  @exception IllegalActionException If there's no director.

	 */



	public void fire() throws IllegalActionException {

		super.fire();

		//TODO take care of loading an ontology

		//TODO open in a Ptolemy effigy

		try {

			UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName()) ;

		} catch (ClassNotFoundException ex) {

			ex.printStackTrace();

		} catch (InstantiationException ex) {

			ex.printStackTrace();

		} catch (IllegalAccessException ex) {

			ex.printStackTrace();

		} catch (UnsupportedLookAndFeelException ex) {

			// TODO Auto-generated catch block

			ex.printStackTrace();

		}

		RenderGrowl renderPolicy =

			new RenderGrowl();

		KRPolicyOWL krPolicy = new KRPolicyOWL();

		GrowlEditor obFrame = new GrowlEditor("Growl v 0.02");

		EditorGUI obPanel = new EditorGUI();

		ESDOntoBrowser esdOntoBrowser =

			new ESDOntoBrowser(

				obFrame,

				obPanel,

				krPolicy,

				renderPolicy,

				new NotificationPolicyOWL(krPolicy));

		renderPolicy.LoadImages();

		//esdOntoBrowsernew.modeSelectPanel.setVisible(true);

		//esdOntoBrowsernew.xmlFileName = null;



        	try {

        		esdOntoBrowser.documentBase = new File(".").toURL();

        		//System.getProperty("user.dir")

        	}

        	catch (Exception ex) {

        	}



		obFrame.addWindowListener(new WindowAdapter() {

			public void windowClosing(WindowEvent e) {

		//		System.exit(0);

			}

		});

		obFrame.getContentPane().setLayout(new java.awt.BorderLayout());

		obFrame.getContentPane().add(  obPanel, java.awt.BorderLayout.CENTER);



		obFrame.setJMenuBar(obPanel.getMainMenuBar());

	   // obFrame.getContentPane().add(obPanel.getJToolBar(), java.awt.BorderLayout.NORTH);

		obFrame.setSize(500, 500);

		obFrame.setVisible(true);

		if (!ontologyFile.getExpression().trim().equals("")) {

			try {

				File loadFile = ontologyFile.asFile();

				obPanel.loadFile(loadFile);

			} catch (Exception ex) {

				ex.printStackTrace();

			}

		}

	}





	public boolean postfire() {

		return false;

	}



	public void wrapup() throws IllegalActionException {

		_noOntoFile = true;

	}



	///////////////////////////////////////////////////////////////////

	////                         private members                   ////



	/** Previous value of ontologyFile parameter. */

	private String _previousOntoFile;



	/** Represents whether no ontology file was specified */

	private boolean _noOntoFile = true;





}

