/* A database connection actor...

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.geon;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.expr.StringParameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.gui.MessageHandler;
import ptolemy.actor.gui.BrowserLauncher;

import ptolemy.data.DBConnectionToken;

import util.*;
//For java APIs
import java.sql.*;
import java.io.*;
import java.util.*;
import java.net.*;
//For XSLT APIs
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.stream.StreamSource;
import javax.xml.transform.stream.StreamResult;

//////////////////////////////////////////////////////////////////////////
//// OpenDBConnection
/**
This actor opens a DB connection and sends a reference to it.
For the mean time the connection is to an MS access DB.

@author Efrat Jaeger
@version $Id: OpenDBConnection.java,v 1.10 2004/08/10 04:59:24 berkley Exp $
@since Ptolemy II 3.0.2
*/
public class OpenDBConnection extends TypedAtomicActor {

 /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */

public OpenDBConnection(CompositeEntity container, String name)
        throws NameDuplicationException, IllegalActionException  {

    super(container, name);

    dbcon = new TypedIOPort(this, "dbcon", false, true);
    // Set the type constraint.
    dbcon.setTypeEquals(BaseType.DBCONNECTION);

    catalog = new StringAttribute(this, "catalog");
    catalog.setExpression("");
//    _catalog = _none;

    driverName = new StringAttribute(this, "driverName");
    databaseURL = new StringAttribute(this, "databaseURL");
    username = new StringParameter(this, "username");
    password = new StringParameter(this, "password");

    _attachText("_iconDescription", "<svg>\n"
            + "<ellipse cx=\"0\" cy=\"-30\" "
            + "rx=\"20\" ry=\"10\"/>\n"
            + "<line x1=\"20\" y1=\"0\" "
            + "x2=\"20\" y2=\"-30\"/>\n"
            + "<line x1=\"-20\" y1=\"0\" "
            + "x2=\"-20\" y2=\"-30\"/>\n"
            + "<line x1=\"-20\" y1=\"0\" "
            + "x2=\"20\" y2=\"0\"/>\n"
            + "</svg>\n");

}

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  /** A reference to a db connection
   */

  public TypedIOPort dbcon;

  public StringAttribute catalog;
  public StringAttribute driverName;
  public StringAttribute databaseURL;
  public StringParameter username;
  public StringParameter password;

  public String strFileOrURL;

  ///////////////////////////////////////////////////////////////////
  ////                        public methods                     ////

  /** Callback for changes in attribute values
   *  Load a dbdriver repository, either GEON or jdbc
   *
   *  @param a The attribute that changed.
   *  @exception IllegalActionException If the offsets array is not
   *   nondecreasing and nonnegative.
   */
  public void attributeChanged(Attribute at) throws IllegalActionException {
     if (at == catalog){
         String cat = catalog.getExpression();
         //FIX ME! should check whether this is GEON or jdbc catalog. For now works only for GEON.
         if (!cat.equals("")) { //if chosen one of the catalogs.
             //use XSLT!
             File xmlFile = new File(
                 "lib/testdata/geon/dbDrivers.xml"); // FIX ME!! RETRIEVE RELATIVE FILE!!
             File xslFile = new File(
                 "lib/testdata/geon/dbDrivers.xsl");

             XSLT XSLTransformer = new XSLT();
             String htmlStr = XSLTransformer.Transform(xmlFile, xslFile);
             System.out.println(htmlStr);

             //use BrowserUI.

             //Generate a universal id for the request
             UUIDGen uuidgen = new UUIDGen();
             String procID = uuidgen.generateUUID();

             //Append the form in the html source with "extraLine" that has the id.
             String extraLine = "<input type=\"hidden\" name=\"ptid\" value=\"" +
                 procID + "\"/>";

             Reader in = new StringReader(htmlStr);

             String fileName = uuidgen.generateUUID();
             File fileToShow = new File(fileName + ".html");

             // THIS CODE ASSUMES THAT THERE WILL BE JUST ONE FORM IN THE HTML!!!
             try {
                 BufferedReader br = new BufferedReader(in);
                 String line;

                 BufferedWriter out = new BufferedWriter(new FileWriter(fileToShow.
                     getPath(), false));
                 System.out.println("path" + fileToShow.getPath()); //REMOVE!!
                 while ( (line = br.readLine()) != null) {
                     int ind = line.toLowerCase().indexOf("</form>");
                     if (ind != -1) {
           //               hasForm = true; //the file has a form. //REMOVE???
                         out.write(line.substring(0, ind) + "\n");

                         out.write(extraLine + "\n");
                         out.write(line.substring(ind) + "\n");
                     }
                     else
                         out.write(line + "\n");
                 }
                 out.close();
                 strFileOrURL = "file://" + fileToShow.getAbsolutePath();
                 br.close();
             }
             catch (IOException e) {
                 if (fileToShow.exists()) {
                   fileToShow.delete();
                 }
                 MessageHandler.error("Error opening file", e);
             }
             try {
               BrowserLauncher.openURL(strFileOrURL);
             }
             catch (Exception ex) {
               if (fileToShow.exists()) {
                 fileToShow.delete();
               }
               MessageHandler.error("Failed to open" + strFileOrURL, ex);
             }

             String fetchURL =
                 "http://geon01.sdsc.edu:8080/pt2/jsp/ptf.jsp?ptid=" + procID;
             System.out.println("fetchURL:  " + fetchURL + "\n");
             try {
                 URL url = new URL(fetchURL);
                 while (true) {
                     HttpURLConnection urlconn = (HttpURLConnection) url.openConnection();
                     urlconn.connect();
                     Thread.sleep(5000);
                     BufferedReader br = new BufferedReader(new InputStreamReader(
                         urlconn.getInputStream()));
                     String line = br.readLine();
                     String resp = "";
                     while (line != null) {
                         System.out.println(line); //REMOVE!!!
                         resp += line + "\n";
                         line = br.readLine();
                     }
                     br.close();
                     urlconn.disconnect();
                     if (resp.trim().toLowerCase().indexOf("<xmp>") != -1) {
               while (!resp.trim().equals("</xmp>")) {
                 int indStart = resp.toLowerCase().indexOf("<name>");
                 int indEnd = resp.toLowerCase().indexOf("</name>");
                 String respLine = resp.substring(indStart + 6, indEnd);
                 String currentParamName = respLine;
                 String respRest = resp.substring(indEnd + 7);
                 resp = respRest;
                 indStart = resp.toLowerCase().indexOf("<value>");
                 indEnd = resp.toLowerCase().indexOf("</value>");
                 respLine = resp.substring(indStart + 7, indEnd);
                 String currentParamValue = respLine;
                 respRest = resp.substring(indEnd + 8);
                 resp = respRest;
                 if (currentParamName.equals("dbURL")) {
                   databaseURL.setExpression(currentParamValue);
                 }
                 else if (currentParamName.equals("driverName")) {
                   driverName.setExpression(currentParamValue);
                 }

 //               System.out.println("RESPONSE LEFT: " + resp); // REMOVE!!!
               }
               break;
             }
             //FIX ME!! DO SOMETHING WITH THE INFORMATION RECEIVED!!
           }
         }
         catch (Exception e) {
           MessageHandler.error("url connection error. ", e);
         }
         catalog.setExpression("");
         fileToShow.delete();
       }
     }
   }

  /** Connect to the Virginia Igneous Rock DB
   * outputs a reference to the DB connection.
   * DriverName = sun.jdbc.odbc.JdbcOdbcDriver
   * url = jdbc:odbc:VirginiaIgneousRock
   */

    public void fire() throws IllegalActionException {
      try {

        String _driverName = driverName.getExpression();
//        System.out.println("driverName >> " + _driverName);
        String _databaseURL = databaseURL.getExpression();
//        System.out.println("databaseURL >> " + _databaseURL);
        String _username = username.stringValue();
//        System.out.println("username >> " + _username);
        String _password = password.stringValue();
//        System.out.println("password >> " + _password);

/*
        // get jdbc driver
        Class.forName("org.objectweb.rmijdbc.Driver").newInstance();

        // get connection
        String url = "jdbc:odbc:IgneousRock";
        String rmiHost = "//klin-pc.sdsc.edu";
        Connection c = DriverManager.getConnection("jdbc:rmi:" + rmiHost + ":38383/" + url, "", "");
        jdbc:rmi://klin-pc.sdsc.edu:38383/jdbc:odbc:IgneousRock
*/

        Class.forName(_driverName).newInstance();

        Connection con = DriverManager.getConnection(_databaseURL,
                                                   _username,
                                                   _password);
        dbcon.broadcast(new DBConnectionToken(con));

      }
      catch (Exception ex) {
        throw new IllegalActionException(this, ex, "fire exception DB connection");
      }
    }

    public boolean postfire() {
      return false;
    }
}