/* A point in polygon actor that also returns the polygon coordinates.



 Copyright (c) 1998-2003 The Regents of the University of California.

 All rights reserved.

 Permission is hereby granted, without written agreement and without

 license or royalty fees, to use, copy, modify, and distribute this

 software and its documentation for any purpose, provided that the above

 copyright notice and the following two paragraphs appear in all copies

 of this software.



 IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

 FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

 ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF

 THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF

 SUCH DAMAGE.



 THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

 INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

 PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF

 CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,

 ENHANCEMENTS, OR MODIFICATIONS.



                                        PT_COPYRIGHT_VERSION_2

                                        COPYRIGHTENDKEY

*/



package org.geon;



import ptolemy.actor.TypedAtomicActor;

import ptolemy.actor.TypedIOPort;

import ptolemy.data.ArrayToken;

import ptolemy.data.BooleanToken;

import ptolemy.data.DoubleToken;

import ptolemy.data.IntToken;

import ptolemy.data.LongToken;

import ptolemy.data.RecordToken;

import ptolemy.data.StringToken;

import ptolemy.data.type.RecordType;

import ptolemy.data.Token;

import ptolemy.data.expr.Parameter;

import ptolemy.data.type.ArrayType;

import ptolemy.data.type.BaseType;

import ptolemy.data.type.Type;

import ptolemy.kernel.CompositeEntity;

import ptolemy.kernel.util.*;



//import java.util.Hashtable;



import java.awt.Polygon;

import java.io.*;

import java.lang.*;

import java.net.*;

import java.util.Vector;



//////////////////////////////////////////////////////////////////////////

//// PointInPolygonXY

/**

This actor receives an array of polygon points and populates them in a polygon

object it also receives a classification point. The actor outputs whether the

point is contained in the polygon along with the polygon coordinates for display

purposes.



@author Efrat Jaeger

@version $Id: PointinPolygonXY.java,v 1.4 2004/05/06 19:19:07 jaeger Exp $

@since Ptolemy II 3.0.2

*/

public class PointinPolygonXY extends TypedAtomicActor {



 /** Construct an actor with the given container and name.

 *  @param container The container.

 *  @param name The name of this actor.

 *  @exception IllegalActionException If the actor cannot be contained

 *   by the proposed container.

 *  @exception NameDuplicationException If the container already has an

 *   actor with this name.

 */

  public PointinPolygonXY(CompositeEntity container, String name)

        throws NameDuplicationException, IllegalActionException  {



      super(container, name);



      // Set the type constraint.

      output = new TypedIOPort(this, "output", false, true); //output (true/false)

      output.setTypeEquals(BaseType.BOOLEAN);

      polygonPoints = new TypedIOPort(this, "polygonPoints", true, false); //Polygon coordinates

      polygonPoints.setTypeEquals(new ArrayType(new ArrayType(BaseType.INT)));

      point = new TypedIOPort(this, "point", true, false); //input point

      point.setTypeEquals(new ArrayType(BaseType.DOUBLE));

      xArray = new TypedIOPort(this, "xArray", false, true);

      xArray.setTypeEquals(new ArrayType(BaseType.INT));

      yArray = new TypedIOPort(this, "yArray", false, true);

      yArray.setTypeEquals(new ArrayType(BaseType.INT));



      _attachText("_iconDescription", "<svg>\n"

                  + "<polygon points=\"-15,-2 0,-15 15,-2 11,15 -11,15\" "

                  + "style=\"fill:white\"/>\n"

                  + "<circle cx=\"3\" cy=\"4\" r=\"1\""

                  + "style=\"fill:black\"/>\n"

                  + "</svg>\n");



  }



  ///////////////////////////////////////////////////////////////////

  ////                     ports and parameters                  ////



  /** An array of polygon points.

   */

  public TypedIOPort polygonPoints;



  /** A sample point.

   */

  public TypedIOPort point;



  /** Specifies whether the point is in the polygon.

   */

  public TypedIOPort output;



  /** The array of x coordinates of the polygon.

   */

  public TypedIOPort xArray;



  /** The array of y coordinates of the polygon.

   */

  public TypedIOPort yArray;



  /** Reads an array of coordinates data and populate the points in a polygon

   *  object.  Returns whether the sample point is contained within the polygon

   *  and the polygon's coordinates arrays.

   *  @exception IllegalActionException If there's no director.

   */

  public void fire() throws IllegalActionException {



      super.fire();

      Polygon P = new Polygon();

      if (polygonPoints.hasToken(0) && point.hasToken(0)) {

        ArrayToken token = (ArrayToken)polygonPoints.get(0);

        arrayLen = (int)(token.length());

        //populate the polygon object.

        for (int i=0; i < arrayLen; i++) {

          x = ((IntToken)(((ArrayToken)token.getElement(i))).getElement(0)).intValue();

          y = ((IntToken)(((ArrayToken)token.getElement(i))).getElement(1)).intValue();

          P.addPoint((int)x,(int)y);

        }

        ArrayToken Point = (ArrayToken)point.get(0);



        //generate the coordinates array tokens.

        Token xTokenArray[] = new Token[arrayLen];

        Token yTokenArray[] = new Token[arrayLen];



        for (int i=0; i < arrayLen; i++) {

          xTokenArray[i] = new IntToken((int)P.xpoints[i]);

          yTokenArray[i] = new IntToken((int)P.ypoints[i]);

        }



        xArray.send(0,new ArrayToken(xTokenArray));

        yArray.send(0,new ArrayToken(yTokenArray));





        Px = ((DoubleToken)Point.getElement(0)).doubleValue();

        Py = ((DoubleToken)Point.getElement(1)).doubleValue();



        String res = new String("{" + Px + "," + Py + "}");



        //classify.

        if (_isPolygonVertex(P,Px,Py)) {

            output.broadcast(BooleanToken.TRUE);

        } else if (_onPolygonEdge(P,Px,Py)) {

            output.broadcast(BooleanToken.TRUE);

        } else if (P.contains(((DoubleToken)Point.getElement(0)).doubleValue(),((DoubleToken)Point.getElement(1)).doubleValue())) {

            output.broadcast(BooleanToken.TRUE);

        } else {

            output.broadcast(BooleanToken.FALSE);

        }

      }

  }



    /** Post fire the actor. Return false to indicated that the

    * process has finished. If it returns true, the process will

    * continue indefinitely.

    */

  public boolean postfire() {

    return true;

  }



  ///////////////////////////////////////////////////////////////////

  ////                         private methods                   ////



  /* Checking whether the given point is one of the polygon's vertices.

   */

  private boolean _isPolygonVertex(Polygon P,double x, double y) {

    for (int i=0; i<arrayLen; i++) {

      if(P.xpoints[i] == x && P.ypoints[i] == y)

        return true;

    }

    return false;

  }



  /* Checking whether the given point is one of the polygon's edges.

   */

  private boolean _onPolygonEdge(Polygon P,double x, double y) {

    int x1,x2,y1,y2;

    for (int i=0; i<arrayLen; i++) {

      x1 = P.xpoints[i];

      x2 = P.xpoints[(i+1)%arrayLen];

      y1 = P.ypoints[i];

      y2 = P.ypoints[(i+1)%arrayLen];

      if (((x1 <= x && x <= x2)||(x2 <= x && x <= x1)) &&    //x and y are between x1,x2 and y1,y2

          ((y1 <= y && y <= y2)||(y2 <= y && y <= y1))) {

        if (((y-y1)/(x-x1)) == ((y-y2)/(x-x2)))

          return true;

      }

    }

    return false;

  }



  ///////////////////////////////////////////////////////////////////

  ////                         private members                   ////



  private int x, y;

  private double Px, Py;

  private int arrayLen;

}

