/* A Digrams actor...

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.geon.test;

import org.geon.*;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.LongToken;
import ptolemy.data.ObjectToken;
import ptolemy.data.RecordToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.RecordType;
import ptolemy.data.type.Type;
import ptolemy.kernel.ComponentEntity;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;

import javax.swing.JOptionPane;

import java.awt.Container;
import java.io.File;

//////////////////////////////////////////////////////////////////////////
//// Diagrams
/**
This is a domain specific actor for processing Rock naming SVG diagrams.
Receive all the diagrams information and the transitions between them from its
predecessor, the DiagramsTransitions actor and a refernce to this level's diagram
and extract the transitions table, the referenced diagram and its coordinates
(For now there are only two digitized diagram).

@author Efrat Jaeger
@version $Id: Diagrams1.java,v 1.6 2004/05/26 02:00:00 jaeger Exp $
@since Ptolemy II 3.0.2
*/
public class Diagrams1 extends TypedAtomicActor {

  /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */
  public Diagrams1(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException
  {
    super(container, name);

    //set the type constraint.
    diagramTransitions = new TypedIOPort(this, "diagramTransitions", true, false);
    diagramTransitions.setTypeEquals(BaseType.OBJECT);
    nextDiagram = new TypedIOPort(this, "nextDiagram", true, false);
    nextDiagram.setTypeEquals(BaseType.INT);
    coordinateNames = new TypedIOPort(this, "coordinateNames", false, true);
    coordinateNames.setTypeEquals(BaseType.OBJECT); // FIX ME: layer for demo purposes!
    transitionTable = new TypedIOPort(this, "transitionTable", false, true);
    transitionTable.setTypeEquals(BaseType.OBJECT);
    diagram = new TypedIOPort(this, "diagram", false, true);
    diagram.setTypeEquals(BaseType.STRING);

    _attachText("_iconDescription", "<svg>\n"
                + "<rect x=\"-25\" y=\"-20\" "
                + "width=\"50\" height=\"40\" "
                + "style=\"fill:yellow\"/>\n"
                + "<polygon points=\"-15,-2 0,-15 15,-2 11,15 -11,15\" "
                + "style=\"fill:white\"/>\n"
            + "</svg>\n");
  }

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  /** All the diagrams and transitions information.
   */
  public TypedIOPort diagramTransitions;

  /** A reference to the diagram to be processed.
   */
  public TypedIOPort nextDiagram;

  /** The coordinates of the referenced diagram.
   */
  public TypedIOPort coordinateNames;

  /** Specifies the transitions between diagrams.
   */
  public TypedIOPort transitionTable; //FIX ME: needs to be implemented as soon as there are more diagrams.

  /** An SVG diagram.
   */
  public TypedIOPort diagram;


  ///////////////////////////////////////////////////////////////////
  ////                         public methods                    ////

  /** Receive diagrams information and a reference to the current diagram.
   *  Output the current diagram, its coordinates names and the transition table.
   *  @exception IllegalActionException If there's no director.
   */
  public void fire() throws IllegalActionException {
//    while (true) { //FIX ME!

      int diagramId = ((IntToken) nextDiagram.get(0)).intValue();
      ObjectToken RSToken = (ObjectToken) diagramTransitions.get(0);
      RockSample1 RS = (RockSample1) RSToken.getValue();
      String idStr = Integer.toString(diagramId);
      //String diagramsPath = RS.getDiagramsPath();
      
      ComponentEntity container = (ComponentEntity)this.getContainer();
      // reach the top level container and read the diagramsPath parameter.
      while (container.getContainer() != null) {
      	container = (ComponentEntity)container.getContainer();
      }
      StringAttribute diagramsPathParam = (StringAttribute) container.getAttribute("diagramsPath");

      String diagramsPath = diagramsPathParam.getExpression(); 
      
      String diagramName = RS.getDiagram(idStr);
      String currDiagram = diagramsPath + "/" + diagramName;
      System.out.println("diagram name = " + currDiagram);
      RS.setCurrentDiagramPath(currDiagram);
      String transFile = RS.getTransFile();
     
//      File diagramFile = new File(currDiagram);
//      GeonXMLUtil parseXML = new GeonXMLUtil();
//      String coordinates = parseXML.getElementsByTagId(diagramFile, "text", "coordinates");
//      System.out.println("coordinates = " + coordinates);

      coordinateNames.broadcast(new ObjectToken(RS));
      //FIX ME: add additional info. (diagram file, etc...)
      diagram.broadcast(new StringToken(currDiagram));
      transitionTable.broadcast(new ObjectToken(RS));

  //  }
  }
}
