/* GetPoint actor...

@Copyright (c) 2002-2003 The Regents of the University of California.
All rights reserved.

Permission is hereby granted, without written agreement and without
license or royalty fees, to use, copy, modify, and distribute this
software and its documentation for any purpose, provided that the
above copyright notice and the following two paragraphs appear in all
copies of this software.

IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
SUCH DAMAGE.

THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,
ENHANCEMENTS, OR MODIFICATIONS.

                                                PT_COPYRIGHT_VERSION 2
                                                COPYRIGHTENDKEY
*/

package org.geon.test;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.data.ArrayToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.DBConnectionToken;
import ptolemy.data.DoubleToken;
import ptolemy.data.IntToken;
import ptolemy.data.ObjectToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.data.type.Type;
import ptolemy.gui.MessageHandler;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;

import org.geon.*;

import java.io.*;
import java.net.*;
import java.lang.*;
import java.util.*;
import java.awt.*;
import java.awt.geom.Point2D;

//////////////////////////////////////////////////////////////////////////
//// GetPoint1
/**
This is a domain specific actor for calculating a calssification point given
mineral composition and coordinate names.

@author Efrat Jaeger
@version $Id: GetPoint1.java,v 1.5 2004/05/26 02:00:00 jaeger Exp $
@since Ptolemy II 3.0.2
*/
public class GetPoint1 extends TypedAtomicActor {

 /** Construct an actor with the given container and name.
 *  @param container The container.
 *  @param name The name of this actor.
 *  @exception IllegalActionException If the actor cannot be contained
 *   by the proposed container.
 *  @exception NameDuplicationException If the container already has an
 *   actor with this name.
 */
  public GetPoint1(CompositeEntity container, String name)
        throws NameDuplicationException, IllegalActionException  {

    super(container, name);

    // Set the type constraint.
    rowInfo = new TypedIOPort(this, "rowInfo", true, false);
    rowInfo.setTypeEquals(BaseType.GENERAL);
    diagramInfo = new TypedIOPort(this, "diagramInfo", true, false);
    diagramInfo.setTypeEquals(BaseType.OBJECT); //FIX ME! for demo purposes!!!
    point = new TypedIOPort(this, "point", false, true);
    point.setTypeEquals(BaseType.GENERAL);
}

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////

  /** Hold the mineral composition info.
   */
  public TypedIOPort rowInfo;

  /** The coordinates/minerals of the classification diagram.
   */
  public TypedIOPort diagramInfo;

  /** The classification point.
   */
  public TypedIOPort point;

  /** Retrieve the mineral composition for the coordinate names and calculate
   *  the classification point.
   */
    public void fire() throws IllegalActionException {

      ObjectToken RSToken = (ObjectToken) diagramInfo.get(0);
      RockSample1 diagramInfo = (RockSample1) RSToken.getValue();
     
      String diagramsPath = diagramInfo.getDiagramsPath();
      String diagramFileStr = diagramInfo.getCurrentDiagramPath();
      System.out.println("current diagram path ==> " + diagramFileStr);
      File svgFile = new File(diagramFileStr);
      System.out.println("file exists? " + svgFile.exists());
      GeonXMLUtil parseXML = new GeonXMLUtil();
      String properties = parseXML.getElementsByTagId(svgFile, "text", "properties");
      System.out.println("properties = " + properties);

      System.out.println("from static");
      System.out.println("leftPadding = " + leftPadding);
      System.out.println("topPadding = " + topPadding);
      System.out.println("diagramWidth" + diagramWidth);
      System.out.println("triangleDiagramHeight" + triangleDiagramHeight);

      int beginInd = 0;
      int endInd = 0;
//TODO: TAKE CARE OF 3 AND 4 VERTICES.???
      beginInd = properties.indexOf("top");
      properties = properties.substring(beginInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("left");

      String topVertex = properties.substring(beginInd+1,endInd-1);

      System.out.println("topVertex = " + topVertex + "...");

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("right");

      String leftVertex = properties.substring(beginInd+1,endInd-1);
      System.out.println("leftVertex = " + leftVertex + "...");

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("bot");

      String rightVertex = properties.substring(beginInd+1,endInd-1);
      System.out.println("rightVertex = " + rightVertex + "...");

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("left");

      String bottomVertex = properties.substring(beginInd+1,endInd-1);
      System.out.println("bottomVertex = " + bottomVertex + "...");

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("top");

      String leftPad = properties.substring(beginInd+1,endInd-1);
      leftPadding = Double.parseDouble(leftPad);

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("diagram");

      String topPad = properties.substring(beginInd+1,endInd-1);
      topPadding = Double.parseDouble(topPad);

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");
      endInd = properties.indexOf("triangle");

      String diagramWth = properties.substring(beginInd+1,endInd-1);
      diagramWidth = Double.parseDouble(diagramWth);

      properties = properties.substring(endInd);
      beginInd = properties.indexOf(" ");

      String triangleWth = properties.substring(beginInd+1);
      triangleDiagramHeight = Double.parseDouble(triangleWth);

      System.out.println("from properties string");
      System.out.println("leftPadding = " + leftPadding + "...");
      System.out.println("topPadding = " + topPadding + "...");
      System.out.println("diagramWidth" + diagramWidth + "...");
      System.out.println("triangleDiagramHeight" + triangleDiagramHeight + "...");

      // FIX ME: for demo purposes. needs to be generalized.
        ObjectToken obj1 = (ObjectToken) rowInfo.get(0);
        RockSample1 mineralComp = (RockSample1) obj1.getValue();
        mineralComp.setAbbrevTable(diagramInfo.getAbbrevTable());
//      while (true) {
//        if (begin) {  // If this is the first sample point in this execution.
            _point = mineralComp.getPointForGabbroOlivine(leftPadding, topPadding, diagramWidth, triangleDiagramHeight,
                                                   topVertex, leftVertex, rightVertex, bottomVertex);
            //FIX ME: Change according to diagram coordinates.
            _addPointToSVG(diagramsPath + "/QAPF.svg",
            		diagramsPath + "/layer1.svg"); //FIX ME!!! relative files
  /*        }
          else if (layer == 2) {
            _point = mineralComp.getPointForGabbroOlivine(leftPadding, topPadding, diagramWidth, triangleDiagramHeight, layer);
            //FIX ME: Change according to diagram coordinates.
            _addPointToSVG(diagramsPath + "/PlagPxHbl.svg",
                           diagramsPath + "/layer2.svg"); //FIX ME!!! relative files
          }
          begin = false;
        } else { //Append the sample point with the previous points in the diagram.
          if (layer == 1) {
            _point = mineralComp.getPointForGabbroOlivine(leftPadding, topPadding, diagramWidth, triangleDiagramHeight, layer);
            //FIX ME: Change according to diagram coordinates.
            _addPointToSVG(diagramsPath + "/layer1.svg",
                           diagramsPath + "/layer1.svg"); //FIX ME!!! relative files
          }
          else if (layer == 2) {
            _point = mineralComp.getPointForGabbroOlivine(leftPadding, topPadding, diagramWidth, triangleDiagramHeight, layer);
            //FIX ME: Change according to diagram coordinates.
            _addPointToSVG(diagramsPath + "/layer2.svg",
                           diagramsPath + "/layer2.svg"); //FIX ME!!! relative files
          }
        }*/
//        System.out.println("point: Px = " + _point.getX() + " , Py = " +
                          // _point.getY());
        point.broadcast(new ObjectToken(_point));
 //     }
    }

    /** Post fire the actor. Return false to indicate that the
    * process has finished. If it returns true, the process will
    * continue indefinitely.
    */
    public boolean postfire() {
      return true;
    }

    public void wrapup() {
      begin = true;
    }
    /** Add the classification point to the SVG file for display purposes.
     */
    private void _addPointToSVG(String inpFile, String outFile) {
      try {
        System.out.println("input : " + inpFile + " , output : " + outFile);
        File input = new File(inpFile);
        BufferedReader br = new BufferedReader(new FileReader(input));
        String line;
        File output = new File(outFile);
        //System.out.println("File name" + file.getName());
        BufferedWriter out = new BufferedWriter(new FileWriter(output));
        //System.out.println("path" + file.getPath());
        String extraLine = "<circle cx='" + _point.getX() +
            "' cy='" + _point.getY() +
            "' r='3' fill='red' stroke='red'/>";
//        System.out.println("Extra line" + extraLine);
        while ( (line = br.readLine()) != null) {
          int ind = line.toLowerCase().indexOf("</svg>");
          if (ind != -1) {
//            System.out.println("Inside extra line");
            out.write(line.substring(0, ind) + "\n");
            out.write(extraLine + "\n");
            out.write(line.substring(ind) + "\n");
          }
          else
            out.write(line + "\n");
        }
        out.close();
        br.close();
      }
      catch (IOException e) {
        MessageHandler.error("Error opening file", e);
      }
    }

    /** The classification point.
     */
    private Point2D.Double _point;
    private boolean begin = true;

    private double width = 420;
    private double height = 700;
    private double leftPadding = 40;
    private double rightPadding = 40;
    private double diagramWidth = width-leftPadding-rightPadding;
    private double triangleDiagramHeight = (diagramWidth * Math.sin(Math.PI/3));
    private double diagramHeight = triangleDiagramHeight * 2;
    private double bottomPadding = 80;
    private double topPadding = height - diagramHeight - bottomPadding;

}