/* * A browser actor...
 *
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.sdm.spa;

import util.UUIDGen;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.IOPort;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.type.BaseType;
import ptolemy.data.StringToken;
import ptolemy.data.*;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.*;
import ptolemy.actor.gui.BrowserLauncher;

// For IO and URL connections
import java.io.*;
import java.net.*;
import java.util.LinkedList;
import java.util.Iterator;
import java.util.List;
import java.net.URLDecoder;
import ptolemy.gui.MessageHandler;
import java.lang.Thread;


//////////////////////////////////////////////////////////////////////////
//// BrowserUI
/**
   BrowserUI actor is a ...

   @author Ilkay Altintas, Efrat Jaeger and Kai Lin
   @version $Id: BrowserUI.java,v 1.29 2004/07/27 01:20:20 jaeger Exp $
*/


public class BrowserUI
    extends TypedAtomicActor {

  /** Construct a BrowserUI actor with the given container and name.
   *  @param container The container.
   *  @param name The name of this actor.
   *  @exception IllegalActionException If the actor cannot be contained
   *   by the proposed container.
   *  @exception NameDuplicationException If the container already has an
   *   actor with this name.
   */
  public BrowserUI(CompositeEntity container, String name) throws
      IllegalActionException, NameDuplicationException {
    super(container, name);

//    fileToShow = new FileParameter(this, "fileToShow");
    //fileToShow.setExpression("c:/tmp.xml");
    configFile = new FileParameter(this, "configFile");
    fileOrURL = new TypedIOPort(this, "fileOrURL", true, false);
    fileOrURL.setTypeEquals(BaseType.STRING);
    new Attribute(fileOrURL, "_showName");
    xmlOutput = new TypedIOPort(this, "xmlOutput", false, true);
    xmlOutput.setTypeEquals(BaseType.STRING);
    new Attribute(xmlOutput, "_showName");
    name1 = new TypedIOPort(this, "name", false, true);
    name1.setTypeEquals(BaseType.STRING);
    new Attribute(name1, "_showName");
    value = new TypedIOPort(this, "value", false, true);
    value.setTypeEquals(BaseType.STRING);
    new Attribute(value, "_showName");

    _attachText("_iconDescription",
                    "<svg>\n"
                    + "<rect x=\"0\" y=\"0\" "
                    + "width=\"100\" height=\"30\" "
                    + "style=\"fill:white\"/>\n"
                    + "<text x=\"10\" y=\"25\""
                    + "style=\"font-size:16; fill:blue; font-family:SansSerif\">"
                    + "BROWSER</text>\n"
                    + "</svg>\n");

  } // end of constructor

  ///////////////////////////////////////////////////////////////////
  ////                     ports and parameters                  ////
//  public FileParameter fileToShow;
  public FileParameter configFile;
  public TypedIOPort fileOrURL;
  public TypedIOPort xmlOutput;
  public TypedIOPort name1;
  public TypedIOPort value;

  ///////////////////////////////////////////////////////////////////
  ////                       public variables                    ////
//  public Process p;
  public Reader in;
  // a flag to verify whether the html contains a form.
  boolean hasForm = false;
  private boolean reFire = true;

  ///////////////////////////////////////////////////////////////////
  ////                        public methods                     ////

  /** Given a ...., BrowserUI actor opens a broser ...
   *
   *  @exception IllegalActionException If there is no director.
   */
  public void fire() throws IllegalActionException {
    try {
      if (fileOrURL.hasToken(0)) {

        //Generate a universal id for the request
        UUIDGen uuidgen = new UUIDGen();
        String procID = uuidgen.generateUUID();

        //Append the form in the html source with "extraLine" that has the id.
        String extraLine = "<input type=\"hidden\" name=\"ptid\" value=\"" +
            procID + "\"/>";

        String strFileToShow = uuidgen.generateUUID();
        strFileToShow = strFileToShow + ".html";
//FIX ME: choose the file extension according to the file type!!!
        File _fileToShow = new File(strFileToShow);
//Define the variable outside!!
/* FIX ME: Need to delete the file if it is generated but an
         exception occurs before the end.
 */

        //If the inputToken is null set reFire to false.
        StringToken inputToken = null;
        try {
          inputToken = (StringToken) fileOrURL.get(0);
        }
        catch (Exception ex) {}

        if (inputToken != null) {
          String inputFileOrURL = inputToken.stringValue();
          String strFileOrURL = inputFileOrURL;
//        System.out.println("STRFILEORURL for string value: >> " + strFileOrURL);
          if (strFileOrURL.trim().equals("")) {
                throw new IllegalActionException(this, "Input string is empty. ");
          }

          if (!strFileOrURL.trim().startsWith("http")) {
            // [An HTML, XML, ot plain text represented as a string]
            if (strFileOrURL.trim().startsWith("<")) { // If the input  a string
              in = new StringReader(strFileOrURL);
            }
            else { // it is a file name
              //System.out.println("strFile" + strFileOrURL);
              int lineEndInd = strFileOrURL.indexOf("\n");
              //System.out.println("CR index: " + lineEndInd);
              if (lineEndInd != -1) { // If the string contains a CR.
                strFileOrURL = strFileOrURL.substring(0, lineEndInd);
              }
              // Remove the 'file:/' string from the file name.
              if (strFileOrURL.trim().startsWith("file://")) {
                strFileOrURL = strFileOrURL.substring(7);
              } else if (strFileOrURL.trim().startsWith("file:/")) {
                strFileOrURL = strFileOrURL.substring(6);
              }
              File file = new File(strFileOrURL.trim());
              //If this is a file name generate the output file in the same
              //directory as the original, as it may use local files.
              String filePath = file.getAbsolutePath();
              int slashInd = filePath.lastIndexOf("/");
              if (slashInd == -1) {
              	slashInd = filePath.lastIndexOf("\\");
              }
              if (slashInd != -1) {
              	strFileToShow = filePath.substring(0,slashInd+1) + strFileToShow;;
                _fileToShow.delete();
                _fileToShow = new File(strFileToShow);
              }
              //System.out.println("file path " + file.getAbsolutePath()+"  "+file.exists());
              in = new FileReader(file);
            }
          }
          else {
            URL url = new URL(strFileOrURL);
            HttpURLConnection urlconn = (HttpURLConnection) url.openConnection();
            in = new InputStreamReader(urlconn.getInputStream());
          }

          // THIS CODE ASSUMES THAT THERE WILL BE JUST ONE FORM IN THE HTML!!!
          try {

            BufferedReader br = new BufferedReader(in);
            String line;

//            File file = fileToShow.asFile();

            //System.out.println("File name" + file.getName());
            BufferedWriter out = new BufferedWriter(new FileWriter(_fileToShow.getAbsolutePath(), false));
            //System.out.println("path" + _fileToShow.getAbsolutePath());
            while ( (line = br.readLine()) != null) {
              int ind = line.toLowerCase().indexOf("</form>");
              if (ind != -1) {
                hasForm = true; //the file has a form.
                out.write(line.substring(0, ind) + "\n");

                out.write(extraLine + "\n");
                out.write(line.substring(ind) + "\n");
              }
              else
                out.write(line + "\n");
            }
            out.close();
            strFileOrURL = "file://" + _fileToShow.getAbsolutePath();
            br.close();
          }
          catch (IOException e) {
            MessageHandler.error("Error opening file", e);
          }

          // if there is no form and it is either an http or a file display the
          // original file.
          if (!hasForm && !(inputFileOrURL.trim().startsWith("<"))) {
            System.out.println("NO FORM!!");
            System.out.println("file to display >>> " + inputFileOrURL);
            BrowserLauncher.openURL(inputFileOrURL);
          }
          else {
            System.out.println("file to display >>> " + strFileOrURL);
            BrowserLauncher.openURL(strFileOrURL);
          }

          String fetchURL =
              "http://geon01.sdsc.edu:8080/pt2/jsp/ptf.jsp?ptid=" +
              procID;
          System.out.println("fetchURL:  " + fetchURL + "\n");
          URL url = new URL(fetchURL);
          //create a function!!

          if (!hasForm) { //if the file doesn't have a form, wait for a browser close.
//            p.waitFor();
            // If the file doen't have a form, it acts like a sink.
            // Therefore, the putput ports are deleted in this case.
            List outPortList = (List)this.outputPortList();
            int numOutPorts = outPortList.size();
//          System.out.println("Number of output ports:::> " + numOutPorts);
            Iterator oports = outPortList.iterator();
            while (oports.hasNext()) {
              IOPort outp = (IOPort) oports.next();
              if (outp.isOutput()) {
                try {
                  outp.setContainer(null);
                }
                catch (Exception e) {}
              }
            }
          }
          else { // wait for user submission.
            while (true) {
              HttpURLConnection urlconn = (HttpURLConnection) url.
                  openConnection();
              urlconn.connect();
              Thread.sleep(5000);
              BufferedReader br = new BufferedReader(new InputStreamReader(
                  urlconn.
                  getInputStream()));
              String line = br.readLine();
              String resp = "";
              while (line != null) {
//              System.out.println(line);
                resp += line + "\n";
                line = br.readLine();
              }
              br.close();
              urlconn.disconnect();

              int xmpInd = resp.toLowerCase().indexOf("<xmp>");
              if (xmpInd != -1) {
                xmlOutput.broadcast(new StringToken(resp));
                resp = resp.substring(xmpInd+5);

                try {

                  while (!resp.trim().equals("</xmp>")) {

                    int indStart = resp.toLowerCase().indexOf("<name>");
                    int indEnd = resp.toLowerCase().indexOf("</name>");
                    String respLine = resp.substring(indStart + 6, indEnd);
                    String currentParamName = respLine;
                    String respRest = resp.substring(indEnd + 7);
                    resp = respRest;
                    indStart = resp.toLowerCase().indexOf("<value>");
                    indEnd = resp.toLowerCase().indexOf("</value>");
                    respLine = resp.substring(indStart + 7, indEnd);
                    String currentParamValue = respLine;
                    respRest = resp.substring(indEnd + 8);

                    name1.broadcast(new StringToken(currentParamName));
                    value.broadcast(new StringToken(currentParamValue));
                        /*                  System.out.println("Current parameter name: " +
                                                         currentParamName);
                         System.out.println("Current parameter value: " +
                                                         currentParamValue);*/

                    List outPortList = (List)this.outputPortList();
                    Iterator oports = outPortList.iterator();
                    while (oports.hasNext()) {
                      TypedIOPort outp = (TypedIOPort) oports.next();
                      String pname = outp.getName();
                      if (pname.equals(currentParamName)) {
                      	// FIXME: add to all port types!!!
                      	if (outp.getType().equals(BaseType.BOOLEAN)) {
                      		outp.broadcast(new BooleanToken(currentParamValue));
                      	}
                      	else {// temporarily all the rest are strings
                      		outp.broadcast(new StringToken(currentParamValue));
                      	}
                      }
                    }
                    resp = respRest;
                  }
                }
                catch (Exception e) {
                  MessageHandler.error(
                      "Error sending the parameter values to ports. ", e);
                }
                break;
              }
            }
          }
        }
        else {
          System.out.println("NoTokenException");
          reFire = false;
        }
        _fileToShow.delete();
      }
    }
    catch (Exception e) {
      MessageHandler.error(
          "Error opening/updating one of the input or parameter files: ", e);
    }
  } // end of fire

  /** Post fire the actor. Return false to indicate that the
   * process has finished. If it returns true, the process will
   * continue indefinitely.
   */
  public boolean postfire() {
    return reFire;
  } // end of postfire

  /** Wrapup the actor. Set reFire to true.
   */
  public void wrapup() {
    reFire = true;
  }


  /** Callback for changes in attribute values.
   *
   *  @param a The attribute that changed.
   *  @exception IllegalActionException If the offsets array is not
   *   nondecreasing and nonnegative.
   */
  public void attributeChanged(ptolemy.kernel.util.Attribute a) throws ptolemy.
      kernel.util.IllegalActionException {
    if ( a == configFile) {
//      System.out.println("in attribute changed: configFile");
      //Token t = ( (ptolemy.data.expr.Variable) a).getToken();
      try {
        File cFile = ( (FileParameter) a).asFile();
        if (cFile != null) {
          //File cFile = t.asFile();
          List outPortList = (List)this.outputPortList();
          Iterator oports = outPortList.iterator();
          while (oports.hasNext()) {
            IOPort outp = (IOPort) oports.next();
            if (outp.isOutput()) {
              try {
                if (! ((outp.getName().equals(name1.getName())) ||
                       (outp.getName().equals(value.getName())) ||
					   (outp.getName().equals(xmlOutput.getName())))) {
//                  System.out.println("NAME OF THE PORT:" + outp.getName());
                  outp.setContainer(null);
                }
                else {
                  _debug("Haven't deleted name and value ports.");
                }
              }
              catch (Exception e) {}
            }
          }

          String line;
          FileReader fr = new FileReader(cFile);
          BufferedReader br = new BufferedReader(fr);
          while ( (line = br.readLine()) != null) {
            int ind = line.toLowerCase().indexOf(" ");
            String portName = line.substring(0, ind);
            String portType = line.substring(ind);
            TypedIOPort pOut = new TypedIOPort(this, portName, false, true);
            // FIXME : Here, instead of creating the portType as string, decode
            //       the 'PortType's entered by the user to Ptolemy types.
            //**** temporary fix for boolean types..
            if (portType.toLowerCase().trim().startsWith("bool")) {
            	pOut.setTypeEquals(BaseType.BOOLEAN);
            } else {// all the rest
            	pOut.setTypeEquals(BaseType.STRING);
            }
            new Attribute(pOut, "_showName");
          }
          fr.close();
          br.close();
        }
      }
      catch (Exception e) {
      	e.printStackTrace();
        _debug("Exception in attributeChanged:::>  " + e);
      }
    }
  } // end of attributeChanged
}