/**
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.sdm.spa;

import java.util.Vector;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.lib.Const;
import ptolemy.data.BooleanToken;
import ptolemy.data.StringToken;
import ptolemy.data.IntToken;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.Workspace;

//////////////////////////////////////////////////////////////////////////
//// Extract Item
/**
Special Actors

@param input: The string that contains the XML content
@param xpath: The xpath for the item to be extracted.

*/

public class EnumHomolog extends TypedAtomicActor {

	/** Construct a constant source with the given container and name.
	 *  Create the <i>value</i> parameter, initialize its value to
	 *  the default value of an IntToken with value 1.
	 *  @param container The container.
	 *  @param name The name of this actor.
	 *  @exception IllegalActionException If the entity cannot be contained
	 *   by the proposed container.
	 *  @exception NameDuplicationException If the container already has an
	 *   actor with this name.
	 */
	public EnumHomolog(CompositeEntity container, String name)
		throws NameDuplicationException, IllegalActionException {
		super(container, name);
		input = new TypedIOPort(this, "input", true, false);
		input.setTypeEquals(BaseType.STRING);
		branchFinished = new TypedIOPort(this, "branchFinished", true, false);
		branchFinished.setTypeEquals(BaseType.BOOLEAN);

		endOfLoop = new TypedIOPort(this, "endOfLoop", false, true);
		endOfLoop.setTypeEquals(BaseType.BOOLEAN);
		WP_GI = new TypedIOPort(this, "GI", false, true);
		WP_GI.setTypeEquals(BaseType.STRING);
		WP_Start = new TypedIOPort(this, "Start", false, true);
		WP_Start.setTypeEquals(BaseType.STRING);
		WP_End = new TypedIOPort(this, "End", false, true);
		WP_End.setTypeEquals(BaseType.STRING);

		NumLimit = new Parameter(this, "NumLimit", new StringToken("0"));

		_attachText(
			"_iconDescription",
			"<svg>\n"
				+ "<rect x=\"0\" y=\"0\" "
				+ "width=\"60\" height=\"20\" "
				+ "style=\"fill:white\"/>\n"
				+ "</svg>\n");
	}

	///////////////////////////////////////////////////////////////////
	////                     ports and parameters                  ////

	/** The value produced by this constant source.
	 *  By default, it contains an IntToken with value 1.  If the
	 *  type of this token is changed during the execution of a model,
	 *  then the director will be asked to redo type resolution.
	 */
	public TypedIOPort input;
	public TypedIOPort branchFinished;
	public TypedIOPort WP_GI;
	public TypedIOPort WP_Start;
	public TypedIOPort WP_End;
	public TypedIOPort endOfLoop;
	public Parameter NumLimit;

	///////////////////////////////////////////////////////////////////
	////                         public methods                    ////

	/** Clone the actor into the specified workspace. This calls the
	 *  base class and then sets the value public variable in the new
	 *  object to equal the cloned parameter in that new object.
	 *  @param workspace The workspace for the new object.
	 *  @return A new actor.
	 *  @exception CloneNotSupportedException If a derived class contains
	 *   an attribute that cannot be cloned.
	 */
	public Object clone(Workspace workspace)
		throws CloneNotSupportedException {
		Const newObject = (Const) super.clone(workspace);
		// Set the type constraint.
		newObject.output.setTypeAtLeast(newObject.value);
		return newObject;
	}

	/** Post fire the actor. Return false to indicated that the
	 *  process has finished. If it returns true, the process will
	 *  continue indefinitely.
	 */
	public boolean postfire() throws IllegalActionException {
		if (_reachedEND) {
			endOfLoop.broadcast(BooleanToken.TRUE);
			return true;
		}
		//endOfLoop.broadcast(BooleanToken.FALSE);
		return true;
	}
	/** Send the token in the <i>value</i> parameter to the output.
	 *  @exception IllegalActionException If it is thrown by the
	 *   send() method sending out the token.
	 */
	public void fire() throws IllegalActionException {
		p_input = ((StringToken) (input.get(0))).stringValue();
		String p_limit = ((StringToken) (NumLimit.getToken())).stringValue();
		int limit = 0;
		try {
			limit = Integer.parseInt(p_limit);
		} catch (Exception e) {
		}

		_debug("EnumHomolog:Limit Number=" + limit);

		//Now we need to extract the first item from the
		//XML content;
		/**
		 * Those are the paths of using Xwrapper
		 String p_xpath_gi = "/resultDoc/output/homolog/genid";
		String p_xpath_start = "/resultDoc/output/homolog/beginMatch";
		String p_xpath_end = "/resultDoc/output/homolog/endMatch";
		*/
		//Following pathes directly query the ncbi blast result.
		String p_cpath  = "/BlastOutput/BlastOutput_iterations/Iteration/Iteration_hits/Hit/";
		String p_xpath_gi = p_cpath + "Hit_id";
		String p_xpath_start = p_cpath + "Hit_hsps/Hsp/Hsp_hit-from";
		String p_xpath_end = p_cpath + "Hit_hsps/Hsp/Hsp_hit-to";;

		Vector items_gi, items_start, items_end;
		XMLUtil xmlutil = new XMLUtil();
		if (p_xpath_gi.startsWith("/")) {
			items_gi = xmlutil.getItems(p_input, p_xpath_gi);
			items_start = xmlutil.getItems(p_input, p_xpath_start);
			items_end = xmlutil.getItems(p_input, p_xpath_end);

			for (int i = 0;
				i < items_gi.size() && (limit == 0 || i < limit);
				i++) {
				String item = (String) items_gi.elementAt(i);

				if (item.length() > 0) {
					//extract the gi #
					//gi|1787566|gb|AE000229.1|AE000229
					_debug(" Broadcast:Process Whole Item:" + item);
					item = item.substring(3);
					int j = item.indexOf("|");
					item = item.substring(0, j);

					_debug("EnumHomolot:Got GI:" + item);
					WP_GI.broadcast(new StringToken(item));
					WP_Start.broadcast(
						new StringToken((String) items_start.elementAt(i)));
					WP_End.broadcast(
						new StringToken((String) items_end.elementAt(i)));

					//Wait until the branch is finished
					boolean tg =
						((BooleanToken) branchFinished.get(0)).booleanValue();
					if (tg)
						_debug("EnumHomolog: Got branch finished trigger.");
				}

			}
			// endOfLoop.broadcast(new IntToken(items_gi.size() < limit ? items_gi.size() : limit ));
		}
		_reachedEND = true;

	}

	private String p_input;
	/** Indicator that we have reached the end of file. */
	private boolean _reachedEND = false;

}
