/**
 *  '$RCSfile: EML2MetadataSpecificationTest.java,v $'
 *  Copyright: 2000 Regents of the University of California and the
 *              National Center for Ecological Analysis and Synthesis
 *    Authors: @authors@
 *    Release: @release@
 *
 *   '$Author: tao $'
 *     '$Date: 2004/08/13 01:02:17 $'
 * '$Revision: 1.4 $'
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

package test.org.ecoinformatics.seek.ecogrid;

import edu.ucsb.nceas.configxml.ConfigXML;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestResult;
import junit.framework.TestSuite;


import org.ecoinformatics.ecogrid.ConditionType;
import org.ecoinformatics.ecogrid.LimitedXPathExpression;
import org.ecoinformatics.ecogrid.OperatorType;
import org.ecoinformatics.ecogrid.QueryType;
import org.ecoinformatics.ecogrid.QueryType_namespace;
import org.ecoinformatics.ecogrid.ResultsetType;
import org.ecoinformatics.seek.ecogrid.EML2MetadataSpecification;
import org.ecoinformatics.seek.ecogrid.ResultRecord;
import org.ecoinformatics.seek.ecogrid.ResultRecordDetail;

import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.StringAttribute;




/**
 * A JUnit test for testing Step class processing
 */
public class EML2MetadataSpecificationTest extends TestCase
{
  ConfigXML config;
  EML2MetadataSpecification eml2Metadata;

  private static final String SEARCHVALUE = "value";
  private static final String ID1         = "foo.1.1";
  private static final String ID2         = "foo.2.1";
  private static final String TITLE1      = "soil1";
  private static final String TITLE2      = "soil2";
  private static final String ENTITY11    = "entity11";
  private static final String ENTITY12    = "entity12";
  private static final String ENTITY21    = "entity21";
  private static final String ENTITY22    = "entity22";
  private static final String ENDPOINT    = "http://dev.nceas.ucsb.edu";

  /**
   * Constructor to build the test
   *
   * @param name the name of the test method
   */
  public EML2MetadataSpecificationTest(String name)
  {
    super(name);
    try
    {
      config = new ConfigXML("lib/ecogridConfigure.xml");
    }
    catch (Exception e)
    {
       fail("Error to initial config file: " + e.getMessage());
    }
    eml2Metadata = new EML2MetadataSpecification();
    eml2Metadata.setConfigure(config);
  }

  /**
   * Establish a testing framework by initializing appropriate objects
   */
  public void setUp()
  {
  }

  /**
   * Release any objects after tests are complete
   */
  public void tearDown()
  {
  }

  /**
   * Create a suite of tests to be run together
   */
  public static Test suite()
  {
    TestSuite suite = new TestSuite();
    suite.addTest(new EML2MetadataSpecificationTest("initialize"));
    suite.addTest(
         new EML2MetadataSpecificationTest("getQuickSearchEcoGridQueryTest"));
    suite.addTest(new EML2MetadataSpecificationTest("transformResultsetTest"));
    return suite;
  }

  /**
   * Run an initial test that always passes to check that the test
   * harness is working.
   */
  public void initialize()
  {
    assertTrue(1 == 1);
  }

  /**
   * The test for method getQuickSearchEcoGridQuery. This method will generate
   * a ecogrid query for quick search
   */
  public void getQuickSearchEcoGridQueryTest()
  {
    try
    {
       QueryType query = eml2Metadata.getQuickSearchEcoGridQuery(SEARCHVALUE);
       String[] tiles = query.getTitle();
       assertTrue(tiles[0].equals("EcoGrid-Search"));
       QueryType_namespace[] namespace = query.getNamespace();
       assertTrue(
          namespace[0].getValue().equals("eml://ecoinformatics.org/eml-2.0.0"));
       String [] returnFields  = query.getReturnfield();
       assertTrue(returnFields[0].equals("dataset/title"));
       assertTrue(returnFields[1].equals("entityName"));
       ConditionType condition = query.getCondition();
       OperatorType operator = condition.getOperator();
       String operatorStr = operator.getValue();
       assertTrue(operatorStr.equals("LIKE"));
       LimitedXPathExpression expression =condition.getConcept();
       String expressionString = expression.getValue();
       assertTrue(expressionString.equals("/"));
       String searchValue = condition.getValue();
       assertTrue(searchValue.equals(SEARCHVALUE));


    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail("Error getQuickSearchEcoGridQueryTest: " + e.getMessage());
    }
  }

  /**
   *  The method to test transformResultset method. The original method will
   *  transform ecogrid resultset to ResultRecord object array which will be
   *  used in kepler
   */
   public void  transformResultsetTest()
   {
     try
     {
       ResultsetType resultType = generateResultSet();
       CompositeEntity resultsRoot = new CompositeEntity();
       resultsRoot.setName("resultset");
       ResultRecord[] resultRecord = eml2Metadata.transformResultset(
           resultType, ENDPOINT, resultsRoot);
       // compare record 1
       ResultRecord record1 = resultRecord[0];
       String name1 = record1.getName();
       assertTrue(name1.equals(TITLE1));
       StringAttribute idAttr1 = (StringAttribute) record1.getAttribute(
           "recordId");
       String id1 = idAttr1.getExpression();
       assertTrue(id1.equals(ID1));
       StringAttribute endPointAttr1 = (StringAttribute)
           record1.getAttribute("endpoint");
       String endpoint1 = endPointAttr1.getExpression();
       assertTrue(endpoint1.equals(ENDPOINT));
       ResultRecordDetail tetail11 = (ResultRecordDetail)
           record1.getAttribute(ENTITY11);
       assertTrue(tetail11.getName().equals(ENTITY11));
       ResultRecordDetail tetail12 = (ResultRecordDetail)
           record1.getAttribute(ENTITY12);
       assertTrue(tetail12.getName().equals(ENTITY12));

       // compare record2
       ResultRecord record2 = resultRecord[1];
       String name2 = record2.getName();
       assertTrue(name2.equals(TITLE2));
       StringAttribute idAttr2 = (StringAttribute) record2.getAttribute(
                                                                   "recordId");
       String id2 = idAttr2.getExpression();
       assertTrue(id2.equals(ID2));
       StringAttribute endPointAttr2 = (StringAttribute)
           record2.getAttribute("endpoint");
       String endpoint2 = endPointAttr2.getExpression();
       assertTrue(endpoint2.equals(ENDPOINT));
       ResultRecordDetail tetail21 = (ResultRecordDetail)
           record2.getAttribute(ENTITY21);
       assertTrue(tetail21.getName().equals(ENTITY21));
       ResultRecordDetail tetail22 = (ResultRecordDetail)
           record2.getAttribute(ENTITY22);
       assertTrue(tetail22.getName().equals(ENTITY22));
     }
     catch (Exception e)
     {
       e.printStackTrace();
       fail("Error getQuickSearchEcoGridQueryTest: " + e.getMessage());
     }

   }

   /*
    * Method to generate a resultset object
    */
   private ResultsetType generateResultSet()
   {
     ResultsetType result = new ResultsetType();
     /*AnyRecordType param1  = new AnyRecordType();
     param1.setIdentifier(ID1);
     param1.setNamespace(EML2MetadataSpecification.EML200NAMESPACE);
     String returnFields1 = "<param name=\"/eml/dataset/title\">" + TITLE1 +
                            "</param>" +
       "<param name=\"/eml/dataset/keywordSet/keyword\">soil survey1</param>" +
       "<param name=\"/eml/dataset/dataTable/EntityGroup/entityName\">"
        + ENTITY11 + "</param>" +
       "<param name=\"/eml/dataset/otherEntity/EntityGroup/entityName\">"
        + ENTITY12 + "</param>";
     param1.setValue(returnFields1);

     AnyRecordType param2  = new AnyRecordType();
     param2.setIdentifier(ID2);
     param2.setNamespace(EML2MetadataSpecification.EML200NAMESPACE);
     String returnFields2 = "<param name=\"/eml/dataset/title\">" + TITLE2 +
                            "</param>" +
     "<param name=\"/eml/dataset/keywordSet/keyword\">soil survey2</param>" +
     "<param name=\"/eml/dataset/dataTable/EntityGroup/entityName\">" + ENTITY21
       + "</param>"+
     "<param name=\"/eml/dataset/otherEntity/EntityGroup/entityName\">"
      + ENTITY22 + "</param>";
     param2.setValue(returnFields2);

     AnyRecordType [] array = new AnyRecordType[2];
     array[0] = param1;
     array[1] = param2;
     result.setRecord(array);*/
     return result;
   }


}
