/**
 *    '$RCSfile: Config.java,v $'
 *
 *     '$Author: higgins $'
 *       '$Date: 2004/08/12 23:40:17 $'
 *   '$Revision: 1.7 $'
 *
 *  For Details: http://kepler.ecoinformatics.org
 *
 * Copyright (c) 2003 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */

package org.ecoinformatics.util;

import java.io.FileNotFoundException;

import javax.xml.transform.TransformerException;

import org.w3c.dom.NodeList;

import edu.ucsb.nceas.configxml.ConfigXML;
import edu.ucsb.nceas.configxml.exception.ElementNotFoundException;

/**
 *
 * configuration file routines
 *
 */
public class Config
{
  private static final String configfilename = "config.xml";

  private ConfigXML config;
  private boolean error;

  /**
   * Constructor for ConfigDatasource.
   * @throws FileNotFoundException
   * @throws Exception
   */
  public Config()
  {
    try
    {
      config = new ConfigXML(configfilename); 
    }
    catch(FileNotFoundException fnfe)
    { 
        throw new RuntimeException("Config file not found at " +
          configfilename +
          " " + fnfe.getMessage());
    }
    catch(Exception e)
    {
      throw new RuntimeException("Unspecified error creating config object: " +
        e.getMessage());
    }
  }

  /**
   *
   */
  public boolean getErrorState()
  {
    return error;
  }

  /**
   * Method getDebugLevel.
   * @return String
   */
  public int getInputStreamBufferSize()
  {
    if( !error ) {
      try {
        return new Integer(config.get("InputStreamBufferSize", 0).trim()).intValue();
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException e)
      {
        System.out.println("You need to add a 'InputStreamBufferSize' element to your " +
          "config file.  InputStreamBufferSize is defaulting to 1024.");
      }
    }
    return 1024;
  }

  /**
   * Method getCatalogPath.
   * @return String
   */
  public String getCatalogPath() throws Exception
  {
    if( !error ) {
      try {
        return config.get("CatalogPath", 0);
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException e)
      {
        throw new Exception("Catalog path element not found");
      }
    }
    return(null);
  }

  /**
   * Method getTempDir.
   * @return String
   */
  public String getTempDir()
  {
    if( !error ) {
      try {
        return config.get("TempDir", 0);
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException e)
      {
        //        System.out.println("TempDir element not found");
      }
    }
    return(null);
  }

  /**
   * Method getSAXParser.
   * @return String
   */
  public String getSAXParser()
  {
   if( !error ) {
      try {
        return config.get("SAXParser", 0);
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException e)
      {
        System.out.println("SAXParser element not found in config file");
      }
    }
    return(null);
  }

  /**
   * Method getHtmlStartPage.
   * @return Object
   */
  public String getHtmlStartPage() throws Exception
  {
    if( !error ) {
      try {
        return config.get("HtmlStartPage", 0);
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException e)
      {
        throw new Exception("HtmlStartPage element not found in config file");
      }
    }
    return("");
  }

  /**
   * Method getLangNodeList.
   * @return NodeList
   * @throws ExecutionException
   */
  public NodeList getLangNodeList()
    throws Exception
  {
    if( !error )
    {
      try
      {
        return config.getPathContent("/config/language");
      }
      catch(Exception e)
      {
        throw new Exception("Could not get language attribute from " +
                                   "the config file.");
      }
    }
    return null;
  }

  /**
   * Method getDataTypes.
   * @return NodeList
   * @throws TransformerException
   */
  public NodeList getDataTypes() throws Exception
  {
    if( !error ) {
      try
      {
        return config.getPathContent("//dataType");
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException enfe)
      {
        throw new Exception("Cannot find any dataType elements " +
          "in the config.xml file: " + enfe.getMessage());
      }

    }
    return null;
  }

  public NodeList getDBdataTypeMappings() throws Exception
  {
    if( !error ) {
      try
      {
        return config.getPathContent("//DBdataTypeMapping/mapping");
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException enfe)
      {
        throw new Exception("Cannot find any data type mapping elements " +
          "in the config.xml file: " + enfe.getMessage());
      }

    }
    return null;

  }

  /**
   * A method to get delimiter mapping nodelist in config file
   * @throws Exception
   * @return NodeList
   */
  public NodeList getDelimiterMappings() throws Exception
  {
    if( !error ) {
      try
      {
        return config.getPathContent("//delimiterMapping/mapping");
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException enfe)
      {
        throw new Exception("Cannot find any delimiter mapping elements " +
          "in the config.xml file: " + enfe.getMessage());
      }

    }
    return null;

  }

  /**
   * returns the lineEnding element
   */
  public String getLineEnding()
  {
    if( !error )
    {
      try
      {
        return config.get("lineEnding", 0);
      }
      catch(edu.ucsb.nceas.configxml.exception.ElementNotFoundException e)
      {
        throw new RuntimeException("lineEnding element not found in " +
           "config.xml file.");
      }
    }
    return "\n";
  }

  /**
   * Method getLocalCatalogPath.
   * @return String
   * @throws Exception
   */
  public String getLocalCatalogPath() throws Exception
  {
    try
    {
      return config.get("local_catalog_path", 0);
    }
    catch(Exception e)
    {
      throw new Exception("Error getting local_catalog_path: " +
                        e.getMessage());
    }
  }

  /**
   * Method save. - pass through to ConfigXML member
   * @throws Exception
   */
  public void save() throws Exception
  {
    config.save();
  }

  /**
   * Method set. - pass through to ConfigXML member
   * @param arg1
   * @param arg2
   * @param arg3
   * @throws ElementNotFoundException
   */
  public void set(String arg1, int arg2, String arg3) throws ElementNotFoundException
  {
    config.set(arg1, arg2, arg3);
  }

}

