/* A dataset actor...



@Copyright (c) 2002-2003 The Regents of the University of California.

All rights reserved.



Permission is hereby granted, without written agreement and without

license or royalty fees, to use, copy, modify, and distribute this

software and its documentation for any purpose, provided that the

above copyright notice and the following two paragraphs appear in all

copies of this software.



IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY

FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES

ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF

THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF

SUCH DAMAGE.



THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,

INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF

MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE

PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF

CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT, UPDATES,

ENHANCEMENTS, OR MODIFICATIONS.



                                                PT_COPYRIGHT_VERSION 2

                                                COPYRIGHTENDKEY

*/



package org.geon.test;



import org.geon.*;



import ptolemy.actor.TypedAtomicActor;

import ptolemy.actor.TypedIOPort;

import ptolemy.data.ArrayToken;

import ptolemy.data.BooleanToken;

import ptolemy.data.DBConnectionToken;

import ptolemy.data.DoubleToken;

import ptolemy.data.IntToken;

import ptolemy.data.ObjectToken;

import ptolemy.data.StringToken;

import ptolemy.data.Token;

import ptolemy.data.type.ArrayType;

import ptolemy.data.type.BaseType;

import ptolemy.data.type.Type;

import ptolemy.kernel.CompositeEntity;

import ptolemy.kernel.util.*;



import java.io.*;

import java.lang.*;

import java.net.*;

import java.sql.*;

import java.util.*;



//////////////////////////////////////////////////////////////////////////

//// DataSet

/**

This is a domain specific actor for populating a rock sample. It receives a row

identifier and an ODBC:JDBC local database name and populates the sample

according to the row content in the database.



@author Efrat Jaeger

@version $Id: DataSet1.java,v 1.1 2004/04/17 01:33:11 jaeger Exp $

@since Ptolemy II 3.0.2

*/

public class DataSet1 extends TypedAtomicActor {



 /** Construct an actor with the given container and name.

 *  @param container The container.

 *  @param name The name of this actor.

 *  @exception IllegalActionException If the actor cannot be contained

 *   by the proposed container.

 *  @exception NameDuplicationException If the container already has an

 *   actor with this name.

 */

public DataSet1(CompositeEntity container, String name)

        throws NameDuplicationException, IllegalActionException  {



    super(container, name);



    // Set the type constraint.

    rowId = new TypedIOPort(this, "rowId", true, false);

    rowId.setTypeEquals(BaseType.INT);

    rowInfo = new TypedIOPort(this, "rowInfo", false, true);

    rowInfo.setTypeEquals(BaseType.GENERAL);

//    databaseName = new StringAttribute(this, "databaseName");

//    databaseName.setExpression("Please insert database name");



    _attachText("_iconDescription", "<svg>\n"

            + "<ellipse cx=\"0\" cy=\"-30\" "

            + "rx=\"20\" ry=\"10\"/>\n"

            + "<line x1=\"20\" y1=\"0\" "

            + "x2=\"20\" y2=\"-30\"/>\n"

            + "<line x1=\"-20\" y1=\"0\" "

            + "x2=\"-20\" y2=\"-30\"/>\n"

            + "<line x1=\"-20\" y1=\"0\" "

            + "x2=\"20\" y2=\"0\"/>\n"

            + "</svg>\n");

}



  ///////////////////////////////////////////////////////////////////

  ////                     ports and parameters                  ////



  /** The input row id for querying the database.

   */

  public TypedIOPort rowId;



  /** The populated Rock information.

   */

  public TypedIOPort rowInfo;



  /** An ODBC:JDBC database name.

   */

//  public StringAttribute databaseName;



  /** Connect to an ODBC:JDBC database and and populate the rock sample dataset.

   *  Returns the dataset wrapped as an object token.

   *  @exception IllegalActionException If there's no director.

   */

    public void fire() throws IllegalActionException {

      try {

        if (rowId.hasToken(0)) {

          int _rowId = ((IntToken)rowId.get(0)).intValue();



//          String dbName = databaseName.getExpression(); //WORKS ONLY FOR ODBC:JDBC



          Class.forName("org.objectweb.rmijdbc.Driver").newInstance();

          String url = "jdbc:rmi://jaeger-pc.sdsc.edu:38383/jdbc:odbc:VirginiaIgneousRock";

          Connection c = DriverManager.getConnection(url,

              "Virginia",

              "Virginia");

          Statement st = c.createStatement();

          String query = "select * from ModalData where ssID = " + _rowId;

          ResultSet rs = st.executeQuery(query);

          ResultSetMetaData md = rs.getMetaData();

          RockSample1 Rock = new RockSample1();



          // Populate a RockSample according to the row info.

          while (rs.next()) {

            for (int i = 1; i <= md.getColumnCount(); i++) {

              String val = rs.getString(i);

              if (val == null) {

                val = "";

              }

              Rock.add(md.getColumnName(i), val);

            }

          }

          rowInfo.broadcast(new ObjectToken(Rock));

          rs.close();

        }

      }

      catch (Exception ex) {

        throw new IllegalActionException(this, ex, "fire exception DB connection");

      }

    }



    /** Return false to indicate that the process has finished.

     *  @exception IllegalActionException If thrown by the super class.

     */

    public boolean postfire() throws IllegalActionException {

      return false;

    }

}