/**
 * Copyright (c) 2004 The Regents of the University of California.
 * All rights reserved.
 *
 * Permission is hereby granted, without written agreement and without
 * license or royalty fees, to use, copy, modify, and distribute this
 * software and its documentation for any purpose, provided that the
 * above copyright notice and the following two paragraphs appear in
 * all copies of this software.
 *
 * IN NO EVENT SHALL THE UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY
 * FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES
 * ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN
 * IF THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
 * PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY
 * OF CALIFORNIA HAS NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,
 * UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
 */
package org.sdm.spa;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.lib.Const;
import ptolemy.data.StringToken;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.Workspace;

//////////////////////////////////////////////////////////////////////////
//// Extract Item
/**
Combine two input strings into one string.
User can specify pre, mid, and post strings
between the two strings.


*/

public class StringConcat extends TypedAtomicActor {

	public StringConcat(CompositeEntity container, String name)
		throws NameDuplicationException, IllegalActionException {
		super(container, name);

		pre = new Parameter(this, "Pre", new StringToken(""));
		mid = new Parameter(this, "Mid", new StringToken(""));
		post = new Parameter(this, "Post", new StringToken(""));

		result = new TypedIOPort(this, "Result", false, true);
		result.setTypeEquals(BaseType.STRING);

		str1 = new TypedIOPort(this, "String 1", true, false);
		str1.setTypeEquals(BaseType.STRING);

		str2 = new TypedIOPort(this, "String 2", true, false);
		str2.setTypeEquals(BaseType.STRING);
		_attachText(
			"_iconDescription",
			"<svg>\n"
				+ "<rect x=\"0\" y=\"0\" "
				+ "width=\"60\" height=\"20\" "
				+ "style=\"fill:white\"/>\n"
				+ "</svg>\n");
	}

	///////////////////////////////////////////////////////////////////
	////                     ports and parameters                  ////

	/** The value produced by this constant source.
	 *  By default, it contains an IntToken with value 1.  If the
	 *  type of this token is changed during the execution of a model,
	 *  then the director will be asked to redo type resolution.
	 */
	public TypedIOPort result;
	public TypedIOPort str1;
	public TypedIOPort str2;
	public Parameter pre;
	public Parameter mid;
	public Parameter post;

	///////////////////////////////////////////////////////////////////
	////                         public methods                    ////

	/** Clone the actor into the specified workspace. This calls the
	 *  base class and then sets the value public variable in the new
	 *  object to equal the cloned parameter in that new object.
	 *  @param workspace The workspace for the new object.
	 *  @return A new actor.
	 *  @exception CloneNotSupportedException If a derived class contains
	 *   an attribute that cannot be cloned.
	 */
	public Object clone(Workspace workspace)
		throws CloneNotSupportedException {
		Const newObject = (Const) super.clone(workspace);
		// Set the type constraint.
		newObject.output.setTypeAtLeast(newObject.value);
		return newObject;
	}

	/** Post fire the actor. Return false to indicated that the
	 *  process has finished. If it returns true, the process will
	 *  continue indefinitely.
	 */
	public boolean postfire() {
		return true;
	}
	/** Send the token in the <i>value</i> parameter to the output.
	 *  @exception IllegalActionException If it is thrown by the
	 *   send() method sending out the token.
	 */
	public void fire() throws IllegalActionException {
		String s1 = ((StringToken) (str1.get(0))).stringValue();
		String s2 = ((StringToken) (str2.get(0))).stringValue();
		String res = ((StringToken)pre.getToken()).stringValue() + s1 + ((StringToken)mid.getToken()).stringValue() + s2 + ((StringToken)post.getToken()).stringValue();
		result.broadcast(new StringToken(res));
	}
}
